import { sharedConfig, untrack, createRenderEffect, createContext, useContext, createSignal, createEffect, onCleanup, createMemo, createComponent, Show, For } from "solid-js";
import { DiffLineType, getSyntaxDiffTemplate, getSyntaxLineTemplate, getPlainDiffTemplate, getPlainLineTemplate, checkDiffLineIncludeChange, SplitSide, composeLen, getSplitContentLines, getUnifiedContentLine, _cacheMap, DiffFile } from "@git-diff-view/core";
export * from "@git-diff-view/core";
import { SplitSide as SplitSide2 } from "@git-diff-view/core";
import { createStore, ref } from "reactivity-store";
function reconcileArrays(parentNode, a, b) {
  let bLength = b.length, aEnd = a.length, bEnd = bLength, aStart = 0, bStart = 0, after = a[aEnd - 1].nextSibling, map = null;
  while (aStart < aEnd || bStart < bEnd) {
    if (a[aStart] === b[bStart]) {
      aStart++;
      bStart++;
      continue;
    }
    while (a[aEnd - 1] === b[bEnd - 1]) {
      aEnd--;
      bEnd--;
    }
    if (aEnd === aStart) {
      const node = bEnd < bLength ? bStart ? b[bStart - 1].nextSibling : b[bEnd - bStart] : after;
      while (bStart < bEnd) parentNode.insertBefore(b[bStart++], node);
    } else if (bEnd === bStart) {
      while (aStart < aEnd) {
        if (!map || !map.has(a[aStart])) a[aStart].remove();
        aStart++;
      }
    } else if (a[aStart] === b[bEnd - 1] && b[bStart] === a[aEnd - 1]) {
      const node = a[--aEnd].nextSibling;
      parentNode.insertBefore(b[bStart++], a[aStart++].nextSibling);
      parentNode.insertBefore(b[--bEnd], node);
      a[aEnd] = b[bEnd];
    } else {
      if (!map) {
        map = /* @__PURE__ */ new Map();
        let i = bStart;
        while (i < bEnd) map.set(b[i], i++);
      }
      const index = map.get(a[aStart]);
      if (index != null) {
        if (bStart < index && index < bEnd) {
          let i = aStart, sequence = 1, t;
          while (++i < aEnd && i < bEnd) {
            if ((t = map.get(a[i])) == null || t !== index + sequence) break;
            sequence++;
          }
          if (sequence > index - bStart) {
            const node = a[aStart];
            while (bStart < index) parentNode.insertBefore(b[bStart++], node);
          } else parentNode.replaceChild(b[bStart++], a[aStart++]);
        } else aStart++;
      } else a[aStart++].remove();
    }
  }
}
const $$EVENTS = "_$DX_DELEGATE";
function template(html, isImportNode, isSVG, isMathML) {
  let node;
  const create = () => {
    const t = document.createElement("template");
    t.innerHTML = html;
    return t.content.firstChild;
  };
  const fn = () => (node || (node = create())).cloneNode(true);
  fn.cloneNode = fn;
  return fn;
}
function delegateEvents(eventNames, document2 = window.document) {
  const e = document2[$$EVENTS] || (document2[$$EVENTS] = /* @__PURE__ */ new Set());
  for (let i = 0, l = eventNames.length; i < l; i++) {
    const name = eventNames[i];
    if (!e.has(name)) {
      e.add(name);
      document2.addEventListener(name, eventHandler);
    }
  }
}
function setAttribute(node, name, value) {
  if (isHydrating(node)) return;
  if (value == null) node.removeAttribute(name);
  else node.setAttribute(name, value);
}
function className(node, value) {
  if (isHydrating(node)) return;
  if (value == null) node.removeAttribute("class");
  else node.className = value;
}
function style(node, value, prev) {
  if (!value) return prev ? setAttribute(node, "style") : value;
  const nodeStyle = node.style;
  if (typeof value === "string") return nodeStyle.cssText = value;
  typeof prev === "string" && (nodeStyle.cssText = prev = void 0);
  prev || (prev = {});
  value || (value = {});
  let v, s;
  for (s in prev) {
    value[s] == null && nodeStyle.removeProperty(s);
    delete prev[s];
  }
  for (s in value) {
    v = value[s];
    if (v !== prev[s]) {
      nodeStyle.setProperty(s, v);
      prev[s] = v;
    }
  }
  return prev;
}
function use(fn, element, arg) {
  return untrack(() => fn(element, arg));
}
function insert(parent, accessor, marker, initial) {
  if (marker !== void 0 && !initial) initial = [];
  if (typeof accessor !== "function") return insertExpression(parent, accessor, initial, marker);
  createRenderEffect((current) => insertExpression(parent, accessor(), current, marker), initial);
}
function isHydrating(node) {
  return !!sharedConfig.context && !sharedConfig.done && (!node || node.isConnected);
}
function eventHandler(e) {
  if (sharedConfig.registry && sharedConfig.events) {
    if (sharedConfig.events.find(([el, ev]) => ev === e)) return;
  }
  let node = e.target;
  const key = `$$${e.type}`;
  const oriTarget = e.target;
  const oriCurrentTarget = e.currentTarget;
  const retarget = (value) => Object.defineProperty(e, "target", {
    configurable: true,
    value
  });
  const handleNode = () => {
    const handler = node[key];
    if (handler && !node.disabled) {
      const data = node[`${key}Data`];
      data !== void 0 ? handler.call(node, data, e) : handler.call(node, e);
      if (e.cancelBubble) return;
    }
    node.host && typeof node.host !== "string" && !node.host._$host && node.contains(e.target) && retarget(node.host);
    return true;
  };
  const walkUpTree = () => {
    while (handleNode() && (node = node._$host || node.parentNode || node.host)) ;
  };
  Object.defineProperty(e, "currentTarget", {
    configurable: true,
    get() {
      return node || document;
    }
  });
  if (sharedConfig.registry && !sharedConfig.done) sharedConfig.done = _$HY.done = true;
  if (e.composedPath) {
    const path = e.composedPath();
    retarget(path[0]);
    for (let i = 0; i < path.length - 2; i++) {
      node = path[i];
      if (!handleNode()) break;
      if (node._$host) {
        node = node._$host;
        walkUpTree();
        break;
      }
      if (node.parentNode === oriCurrentTarget) {
        break;
      }
    }
  } else walkUpTree();
  retarget(oriTarget);
}
function insertExpression(parent, value, current, marker, unwrapArray) {
  const hydrating = isHydrating(parent);
  if (hydrating) {
    !current && (current = [...parent.childNodes]);
    let cleaned = [];
    for (let i = 0; i < current.length; i++) {
      const node = current[i];
      if (node.nodeType === 8 && node.data.slice(0, 2) === "!$") node.remove();
      else cleaned.push(node);
    }
    current = cleaned;
  }
  while (typeof current === "function") current = current();
  if (value === current) return current;
  const t = typeof value, multi = marker !== void 0;
  parent = multi && current[0] && current[0].parentNode || parent;
  if (t === "string" || t === "number") {
    if (hydrating) return current;
    if (t === "number") {
      value = value.toString();
      if (value === current) return current;
    }
    if (multi) {
      let node = current[0];
      if (node && node.nodeType === 3) {
        node.data !== value && (node.data = value);
      } else node = document.createTextNode(value);
      current = cleanChildren(parent, current, marker, node);
    } else {
      if (current !== "" && typeof current === "string") {
        current = parent.firstChild.data = value;
      } else current = parent.textContent = value;
    }
  } else if (value == null || t === "boolean") {
    if (hydrating) return current;
    current = cleanChildren(parent, current, marker);
  } else if (t === "function") {
    createRenderEffect(() => {
      let v = value();
      while (typeof v === "function") v = v();
      current = insertExpression(parent, v, current, marker);
    });
    return () => current;
  } else if (Array.isArray(value)) {
    const array = [];
    const currentArray = current && Array.isArray(current);
    if (normalizeIncomingArray(array, value, current, unwrapArray)) {
      createRenderEffect(() => current = insertExpression(parent, array, current, marker, true));
      return () => current;
    }
    if (hydrating) {
      if (!array.length) return current;
      if (marker === void 0) return current = [...parent.childNodes];
      let node = array[0];
      if (node.parentNode !== parent) return current;
      const nodes = [node];
      while ((node = node.nextSibling) !== marker) nodes.push(node);
      return current = nodes;
    }
    if (array.length === 0) {
      current = cleanChildren(parent, current, marker);
      if (multi) return current;
    } else if (currentArray) {
      if (current.length === 0) {
        appendNodes(parent, array, marker);
      } else reconcileArrays(parent, current, array);
    } else {
      current && cleanChildren(parent);
      appendNodes(parent, array);
    }
    current = array;
  } else if (value.nodeType) {
    if (hydrating && value.parentNode) return current = multi ? [value] : value;
    if (Array.isArray(current)) {
      if (multi) return current = cleanChildren(parent, current, marker, value);
      cleanChildren(parent, current, null, value);
    } else if (current == null || current === "" || !parent.firstChild) {
      parent.appendChild(value);
    } else parent.replaceChild(value, parent.firstChild);
    current = value;
  } else ;
  return current;
}
function normalizeIncomingArray(normalized, array, current, unwrap) {
  let dynamic = false;
  for (let i = 0, len = array.length; i < len; i++) {
    let item = array[i], prev = current && current[normalized.length], t;
    if (item == null || item === true || item === false) ;
    else if ((t = typeof item) === "object" && item.nodeType) {
      normalized.push(item);
    } else if (Array.isArray(item)) {
      dynamic = normalizeIncomingArray(normalized, item, prev) || dynamic;
    } else if (t === "function") {
      if (unwrap) {
        while (typeof item === "function") item = item();
        dynamic = normalizeIncomingArray(
          normalized,
          Array.isArray(item) ? item : [item],
          Array.isArray(prev) ? prev : [prev]
        ) || dynamic;
      } else {
        normalized.push(item);
        dynamic = true;
      }
    } else {
      const value = String(item);
      if (prev && prev.nodeType === 3 && prev.data === value) normalized.push(prev);
      else normalized.push(document.createTextNode(value));
    }
  }
  return dynamic;
}
function appendNodes(parent, array, marker = null) {
  for (let i = 0, len = array.length; i < len; i++) parent.insertBefore(array[i], marker);
}
function cleanChildren(parent, current, marker, replacement) {
  if (marker === void 0) return parent.textContent = "";
  const node = replacement || document.createTextNode("");
  if (current.length) {
    let inserted = false;
    for (let i = current.length - 1; i >= 0; i--) {
      const el = current[i];
      if (node !== el) {
        const isParent = el.parentNode === parent;
        if (!inserted && !i)
          isParent ? parent.replaceChild(node, el) : parent.insertBefore(node, marker);
        else isParent && el.remove();
      } else inserted = true;
    }
  } else parent.insertBefore(node, marker);
  return [node];
}
function __classPrivateFieldGet(receiver, state, kind, f) {
  if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
  if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
  return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
}
function __classPrivateFieldSet(receiver, state, value, kind, f) {
  if (typeof state === "function" ? receiver !== state || true : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
  return state.set(receiver, value), value;
}
typeof SuppressedError === "function" ? SuppressedError : function(error, suppressed, message) {
  var e = new Error(message);
  return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};
var _TextMeasure_instances, _TextMeasure_key, _TextMeasure_map, _TextMeasure_getInstance;
let canvasCtx = null;
const getKey = (font, text) => {
  return `${font.fontFamily}-${font.fontStyle}-${font.fontSize}-${text}`;
};
const getStableKey = (font, text) => {
  return getKey(font, "0".repeat(text.length));
};
class TextMeasure {
  constructor() {
    _TextMeasure_instances.add(this);
    _TextMeasure_key.set(this, "");
    _TextMeasure_map.set(this, {});
  }
  measure(text, font) {
    const currentKey = getStableKey(font, text);
    if (__classPrivateFieldGet(this, _TextMeasure_map, "f")[currentKey]) {
      return __classPrivateFieldGet(this, _TextMeasure_map, "f")[currentKey];
    }
    const instance2 = __classPrivateFieldGet(this, _TextMeasure_instances, "m", _TextMeasure_getInstance).call(this);
    if (font) {
      const currentFontKey = `${font.fontFamily}-${font.fontStyle}-${font.fontSize}`;
      if (__classPrivateFieldGet(this, _TextMeasure_key, "f") !== currentFontKey) {
        __classPrivateFieldSet(this, _TextMeasure_key, currentFontKey);
        instance2.font = `${font.fontStyle || ""} ${font.fontSize || ""} ${font.fontFamily || ""}`;
      }
    } else {
      instance2.font = "";
    }
    const textWidth = instance2.measureText(text).width;
    return textWidth;
  }
}
_TextMeasure_key = /* @__PURE__ */ new WeakMap(), _TextMeasure_map = /* @__PURE__ */ new WeakMap(), _TextMeasure_instances = /* @__PURE__ */ new WeakSet(), _TextMeasure_getInstance = function _TextMeasure_getInstance2() {
  canvasCtx = canvasCtx || document.createElement("canvas").getContext("2d");
  return canvasCtx;
};
let instance = null;
const getTextMeasureInstance = () => {
  instance = instance || new TextMeasure();
  return instance;
};
const addContentBGName = "--diff-add-content--";
const delContentBGName = "--diff-del-content--";
const borderColorName = "--diff-border--";
const addLineNumberBGName = "--diff-add-lineNumber--";
const delLineNumberBGName = "--diff-del-lineNumber--";
const plainContentBGName = "--diff-plain-content--";
const expandContentBGName = "--diff-expand-content--";
const plainLineNumberColorName = "--diff-plain-lineNumber-color--";
const expandLineNumberColorName = "--diff-expand-lineNumber-color--";
const plainLineNumberBGName = "--diff-plain-lineNumber--";
const expandLineNumberBGName = "--diff-expand-lineNumber--";
const hunkContentBGName = "--diff-hunk-content--";
const hunkContentColorName = "--diff-hunk-content-color--";
const hunkLineNumberBGName = "--diff-hunk-lineNumber--";
const addContentHighlightBGName = "--diff-add-content-highlight--";
const delContentHighlightBGName = "--diff-del-content-highlight--";
const addWidgetBGName = "--diff-add-widget--";
const addWidgetColorName = "--diff-add-widget-color--";
const emptyBGName = "--diff-empty-content--";
const getContentBG = (isAdded, isDelete, hasDiff) => {
  return isAdded ? `var(${addContentBGName})` : isDelete ? `var(${delContentBGName})` : hasDiff ? `var(${plainContentBGName})` : `var(${expandContentBGName})`;
};
const getLineNumberBG = (isAdded, isDelete, hasDiff) => {
  return isAdded ? `var(${addLineNumberBGName})` : isDelete ? `var(${delLineNumberBGName})` : hasDiff ? `var(${plainLineNumberBGName})` : `var(${expandLineNumberBGName})`;
};
const removeAllSelection = () => {
  const selection = window.getSelection();
  for (let i = 0; i < selection.rangeCount; i++) {
    selection.removeRange(selection.getRangeAt(i));
  }
};
const syncScroll = (left, right) => {
  const onScroll = function(event) {
    if (event === null || event.target === null)
      return;
    if (event.target === left) {
      right.scrollTop = left.scrollTop;
      right.scrollLeft = left.scrollLeft;
    } else {
      left.scrollTop = right.scrollTop;
      left.scrollLeft = right.scrollLeft;
    }
  };
  if (!left.onscroll) {
    left.onscroll = onScroll;
  }
  if (!right.onscroll) {
    right.onscroll = onScroll;
  }
  return () => {
    left.onscroll = null;
    right.onscroll = null;
  };
};
const diffFontSizeName = "--diff-font-size--";
const diffAsideWidthName = "--diff-aside-width--";
var NewLineSymbol;
(function(NewLineSymbol2) {
  NewLineSymbol2[NewLineSymbol2["CRLF"] = 1] = "CRLF";
  NewLineSymbol2[NewLineSymbol2["CR"] = 2] = "CR";
  NewLineSymbol2[NewLineSymbol2["LF"] = 3] = "LF";
  NewLineSymbol2[NewLineSymbol2["NEWLINE"] = 4] = "NEWLINE";
  NewLineSymbol2[NewLineSymbol2["NORMAL"] = 5] = "NORMAL";
  NewLineSymbol2[NewLineSymbol2["NULL"] = 6] = "NULL";
})(NewLineSymbol || (NewLineSymbol = {}));
const getSymbol = (symbol) => {
  switch (symbol) {
    case NewLineSymbol.LF:
      return "␊";
    case NewLineSymbol.CR:
      return "␍";
    case NewLineSymbol.CRLF:
      return "␍␊";
    default:
      return "";
  }
};
var DiffModeEnum;
(function(DiffModeEnum2) {
  DiffModeEnum2[DiffModeEnum2["SplitGitHub"] = 1] = "SplitGitHub";
  DiffModeEnum2[DiffModeEnum2["SplitGitLab"] = 2] = "SplitGitLab";
  DiffModeEnum2[DiffModeEnum2["Split"] = 3] = "Split";
  DiffModeEnum2[DiffModeEnum2["Unified"] = 4] = "Unified";
})(DiffModeEnum || (DiffModeEnum = {}));
const DiffViewContext = createContext();
const useDiffViewContext = () => useContext(DiffViewContext);
const generateHook = (key) => {
  return () => {
    const reactiveHook = useDiffViewContext();
    const [state, setState] = createSignal(reactiveHook == null ? void 0 : reactiveHook.getReadonlyState()[key]);
    createEffect(() => {
      const init = () => setState(() => reactiveHook == null ? void 0 : reactiveHook.getReadonlyState()[key]);
      init();
      const unsubscribe = reactiveHook == null ? void 0 : reactiveHook.subscribe(
        (s) => s[key],
        () => init()
      );
      onCleanup(() => unsubscribe == null ? void 0 : unsubscribe());
    });
    return state;
  };
};
const useEnableWrap = generateHook("enableWrap");
const useFontSize = generateHook("fontSize");
const useIsMounted = generateHook("isMounted");
const useTextWidth = ({
  text,
  font
}) => {
  var _a;
  const isMounted = useIsMounted();
  let baseSize = 6;
  const fontSize = parseInt(((_a = font == null ? void 0 : font()) == null ? void 0 : _a.fontSize) || "14");
  baseSize += fontSize > 10 ? (fontSize - 10) * 0.6 : 0;
  const [width, setWidth] = createSignal(baseSize * text().length);
  const measureText = () => {
    const isMountedValue = isMounted();
    if (!isMountedValue) return;
    setWidth(getTextMeasureInstance().measure(text(), (font == null ? void 0 : font()) || {}));
  };
  createEffect(measureText);
  return width;
};
const useMode = generateHook("mode");
const useId = generateHook("id");
const useDomWidth = ({ selector, enable }) => {
  const id = useId();
  const mounted = useIsMounted();
  const [width, setWidth] = createSignal(0);
  const observeWidth = () => {
    if (!mounted()) return;
    if (enable()) {
      const container = document.querySelector(`#diff-root${id()}`);
      const wrapper = container == null ? void 0 : container.querySelector(selector());
      if (!wrapper) return;
      const typedWrapper = wrapper;
      const cb = () => {
        const rect = wrapper == null ? void 0 : wrapper.getBoundingClientRect();
        setWidth((rect == null ? void 0 : rect.width) ?? 0);
      };
      cb();
      const cleanCb = () => {
        var _a, _b, _c;
        (_a = typedWrapper == null ? void 0 : typedWrapper.__observeCallback) == null ? void 0 : _a.delete(cb);
        if (((_b = typedWrapper == null ? void 0 : typedWrapper.__observeCallback) == null ? void 0 : _b.size) === 0) {
          (_c = typedWrapper.__observeInstance) == null ? void 0 : _c.disconnect();
          typedWrapper.removeAttribute("data-observe");
          delete typedWrapper.__observeCallback;
          delete typedWrapper.__observeInstance;
        }
      };
      if (typedWrapper.__observeCallback) {
        typedWrapper.__observeCallback.add(cb);
        onCleanup(() => cleanCb());
        return;
      }
      typedWrapper.__observeCallback = /* @__PURE__ */ new Set();
      typedWrapper.__observeCallback.add(cb);
      const observer = new ResizeObserver(() => {
        var _a, _b;
        return (_b = (_a = typedWrapper == null ? void 0 : typedWrapper.__observeCallback) == null ? void 0 : _a.forEach) == null ? void 0 : _b.call(_a, (cb2) => cb2());
      });
      typedWrapper.__observeInstance = observer;
      observer.observe(typedWrapper);
      typedWrapper.setAttribute("data-observe", "height");
      onCleanup(() => cleanCb());
    }
  };
  createEffect(observeWidth);
  return width;
};
const useSyncHeight = ({
  selector,
  wrapper,
  side,
  enable
}) => {
  const id = useId();
  const isMounted = useIsMounted();
  const observeHeight = () => {
    if (!isMounted()) return;
    if (enable()) {
      let clean = () => {
      };
      const container = document.querySelector(`#diff-root${id()}`);
      const elements = Array.from((container == null ? void 0 : container.querySelectorAll(selector())) || []);
      const wrappers = wrapper() ? Array.from((container == null ? void 0 : container.querySelectorAll(wrapper())) || []) : elements;
      if (elements.length === 2 && wrappers.length === 2) {
        const ele1 = elements[0];
        const ele2 = elements[1];
        const wrapper1 = wrappers[0];
        const wrapper2 = wrappers[1];
        const target = ele1.getAttribute("data-side") === side() ? ele1 : ele2;
        const typedTarget = target;
        const cb = () => {
          ele1.style.height = "auto";
          ele2.style.height = "auto";
          const rect1 = ele1.getBoundingClientRect();
          const rect2 = ele2.getBoundingClientRect();
          const maxHeight = Math.max(rect1.height, rect2.height);
          wrapper1.style.height = maxHeight + "px";
          wrapper2.style.height = maxHeight + "px";
          wrapper1.setAttribute("data-sync-height", String(maxHeight));
          wrapper2.setAttribute("data-sync-height", String(maxHeight));
        };
        cb();
        const cleanCb = () => {
          var _a, _b, _c;
          (_a = typedTarget.__observeCallback) == null ? void 0 : _a.delete(cb);
          if (((_b = typedTarget.__observeCallback) == null ? void 0 : _b.size) === 0) {
            (_c = typedTarget.__observeInstance) == null ? void 0 : _c.disconnect();
            target.removeAttribute("data-observe");
            delete typedTarget.__observeCallback;
            delete typedTarget.__observeInstance;
          }
        };
        if (typedTarget.__observeCallback) {
          typedTarget.__observeCallback.add(cb);
          clean = cleanCb;
          return;
        }
        typedTarget.__observeCallback = /* @__PURE__ */ new Set();
        typedTarget.__observeCallback.add(cb);
        const observer = new ResizeObserver(() => {
          var _a;
          return (_a = typedTarget.__observeCallback) == null ? void 0 : _a.forEach((cb2) => cb2());
        });
        typedTarget.__observeInstance = observer;
        observer.observe(target);
        target.setAttribute("data-observe", "height");
        clean = cleanCb;
      }
      onCleanup(() => clean());
    }
  };
  createEffect(observeHeight);
};
const useExtendData = generateHook("extendData");
const useRenderExtend = generateHook("renderExtendLine");
const useRenderWidget = generateHook("renderWidgetLine");
const useEnableAddWidget = generateHook("enableAddWidget");
const useEnableHighlight = generateHook("enableHighlight");
const useOnAddWidgetClick = generateHook("onAddWidgetClick");
var _tmpl$$j = /* @__PURE__ */ template(`<div><button class="diff-add-widget z-[1] flex h-full w-full origin-center cursor-pointer items-center justify-center rounded-md text-[1.2em]">+`), _tmpl$2$e = /* @__PURE__ */ template(`<div class="diff-add-widget-wrapper invisible absolute left-[100%] top-[1px] translate-x-[-50%] select-none transition-transform hover:scale-110 group-hover:visible"><button class="diff-add-widget z-[1] flex h-full w-full origin-center cursor-pointer items-center justify-center rounded-md text-[1.2em]">+`);
const DiffSplitAddWidget = (props) => {
  return (() => {
    var _el$ = _tmpl$$j(), _el$2 = _el$.firstChild;
    `calc(var(${diffFontSizeName}) * 1.4)` != null ? _el$.style.setProperty("width", `calc(var(${diffFontSizeName}) * 1.4)`) : _el$.style.removeProperty("width");
    `calc(var(${diffFontSizeName}) * 1.4)` != null ? _el$.style.setProperty("height", `calc(var(${diffFontSizeName}) * 1.4)`) : _el$.style.removeProperty("height");
    _el$2.$$click = () => {
      var _a, _b, _c;
      props.onOpenAddWidget(props.lineNumber, props.side);
      (_c = (_b = (_a = props.onWidgetClick) == null ? void 0 : _a.call(props)) == null ? void 0 : _b.current) == null ? void 0 : _c.call(_b, props.lineNumber, props.side);
    };
    `var(${addWidgetColorName})` != null ? _el$2.style.setProperty("color", `var(${addWidgetColorName})`) : _el$2.style.removeProperty("color");
    `var(${addWidgetBGName})` != null ? _el$2.style.setProperty("background-color", `var(${addWidgetBGName})`) : _el$2.style.removeProperty("background-color");
    createRenderEffect(() => className(_el$, "diff-add-widget-wrapper invisible select-none transition-transform hover:scale-110 group-hover:visible" + (props.class ? " " + props.class : "")));
    return _el$;
  })();
};
const DiffUnifiedAddWidget = (props) => {
  return (() => {
    var _el$3 = _tmpl$2$e(), _el$4 = _el$3.firstChild;
    `calc(var(${diffFontSizeName}) * 1.4)` != null ? _el$3.style.setProperty("width", `calc(var(${diffFontSizeName}) * 1.4)`) : _el$3.style.removeProperty("width");
    `calc(var(${diffFontSizeName}) * 1.4)` != null ? _el$3.style.setProperty("height", `calc(var(${diffFontSizeName}) * 1.4)`) : _el$3.style.removeProperty("height");
    _el$4.$$click = () => {
      var _a, _b, _c;
      props.onOpenAddWidget(props.lineNumber, props.side);
      (_c = (_b = (_a = props.onWidgetClick) == null ? void 0 : _a.call(props)) == null ? void 0 : _b.current) == null ? void 0 : _c.call(_b, props.lineNumber, props.side);
    };
    `var(${addWidgetColorName})` != null ? _el$4.style.setProperty("color", `var(${addWidgetColorName})`) : _el$4.style.removeProperty("color");
    `var(${addWidgetBGName})` != null ? _el$4.style.setProperty("background-color", `var(${addWidgetBGName})`) : _el$4.style.removeProperty("background-color");
    return _el$3;
  })();
};
delegateEvents(["click"]);
var _tmpl$$i = /* @__PURE__ */ template(`<svg aria-label="No newline at end of file"role=img viewBox="0 0 16 16"version=1.1 fill=currentColor><path d="M4.25 7.25a.75.75 0 0 0 0 1.5h7.5a.75.75 0 0 0 0-1.5h-7.5Z"></path><path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0Zm-1.5 0a6.5 6.5 0 1 0-13 0 6.5 6.5 0 0 0 13 0Z">`);
const DiffNoNewLine = () => {
  return _tmpl$$i();
};
var _tmpl$$h = /* @__PURE__ */ template(`<span class=diff-line-content-raw><span data-template>`), _tmpl$2$d = /* @__PURE__ */ template(`<span class=diff-line-content-raw>`), _tmpl$3$a = /* @__PURE__ */ template(`<span class=diff-line-content-raw><span><span data-diff-highlight class=rounded-[0.2em]>`), _tmpl$4$7 = /* @__PURE__ */ template(`<span data-newline-symbol>`), _tmpl$5$6 = /* @__PURE__ */ template(`<span data-no-newline-at-end-of-file-symbol>`), _tmpl$6$4 = /* @__PURE__ */ template(`<span class=diff-line-syntax-raw><span data-template>`), _tmpl$7$4 = /* @__PURE__ */ template(`<span class=diff-line-syntax-raw>`), _tmpl$8$2 = /* @__PURE__ */ template(`<span>`), _tmpl$9$1 = /* @__PURE__ */ template(`<span class=diff-line-syntax-raw><span>`), _tmpl$10$1 = /* @__PURE__ */ template(`<span><span data-diff-highlight>`), _tmpl$11$1 = /* @__PURE__ */ template(`<div class="diff-line-content-item pl-[2.0em]"><span class="diff-line-content-operator ml-[-1.5em] inline-block w-[1.5em] select-none indent-[0.2em]">`);
const DiffString = (props) => {
  const getIsNewLineSymbolChanged = () => {
    var _a, _b;
    return (_b = (_a = props.diffLine) == null ? void 0 : _a.changes) == null ? void 0 : _b.newLineSymbol;
  };
  const initTemplateIfNeed = () => {
    var _a;
    if (props.enableTemplate && props.operator && props.diffLine && !((_a = props.diffLine) == null ? void 0 : _a.plainTemplate) && typeof getPlainDiffTemplate === "function") {
      getPlainDiffTemplate({
        diffLine: props.diffLine,
        rawLine: props.rawLine,
        operator: props.operator
      });
    }
    if (props.enableTemplate && props.plainLine && !props.plainLine.template) {
      props.plainLine.template = getPlainLineTemplate(props.plainLine.value);
    }
  };
  createMemo(initTemplateIfNeed);
  return createComponent(Show, {
    get when() {
      var _a, _b;
      return (_b = (_a = props.diffLine) == null ? void 0 : _a.changes) == null ? void 0 : _b.hasLineChange;
    },
    get fallback() {
      return createComponent(Show, {
        get when() {
          var _a;
          return props.enableTemplate && ((_a = props.plainLine) == null ? void 0 : _a.template);
        },
        get fallback() {
          return (() => {
            var _el$5 = _tmpl$2$d();
            insert(_el$5, () => props.rawLine);
            return _el$5;
          })();
        },
        get children() {
          var _el$3 = _tmpl$$h(), _el$4 = _el$3.firstChild;
          createRenderEffect(() => {
            var _a;
            return _el$4.innerHTML = (_a = props.plainLine) == null ? void 0 : _a.template;
          });
          return _el$3;
        }
      });
    },
    get children() {
      return createComponent(Show, {
        get when() {
          var _a;
          return props.enableTemplate && ((_a = props.diffLine) == null ? void 0 : _a.plainTemplate);
        },
        get fallback() {
          return function() {
            var _a, _b;
            const range = (_b = (_a = props.diffLine) == null ? void 0 : _a.changes) == null ? void 0 : _b.range;
            const str1 = props.rawLine.slice(0, range == null ? void 0 : range.location);
            const str2 = props.rawLine.slice(range == null ? void 0 : range.location, ((range == null ? void 0 : range.location) || 0) + ((range == null ? void 0 : range.length) || 0));
            const str3 = props.rawLine.slice(((range == null ? void 0 : range.location) || 0) + ((range == null ? void 0 : range.length) || 0));
            const isLast = str2.includes("\n");
            const _str2 = isLast ? str2.replace("\n", "").replace("\r", "") : str2;
            return (() => {
              var _el$6 = _tmpl$3$a(), _el$7 = _el$6.firstChild, _el$8 = _el$7.firstChild;
              insert(_el$7, str1, _el$8);
              insert(_el$8, isLast ? [_str2, (() => {
                var _el$9 = _tmpl$4$7();
                insert(_el$9, () => getSymbol(getIsNewLineSymbolChanged()));
                return _el$9;
              })()] : str2);
              insert(_el$7, str3, null);
              insert(_el$6, (() => {
                var _c$2 = createMemo(() => getIsNewLineSymbolChanged() === NewLineSymbol.NEWLINE);
                return () => _c$2() && (() => {
                  var _el$10 = _tmpl$5$6();
                  `var(${diffFontSizeName})` != null ? _el$10.style.setProperty("width", `var(${diffFontSizeName})`) : _el$10.style.removeProperty("width");
                  `var(${diffFontSizeName})` != null ? _el$10.style.setProperty("height", `var(${diffFontSizeName})`) : _el$10.style.removeProperty("height");
                  insert(_el$10, createComponent(DiffNoNewLine, {}));
                  createRenderEffect(() => className(_el$10, props.enableWrap ? "block !text-red-500" : "inline-block align-middle !text-red-500"));
                  return _el$10;
                })();
              })(), null);
              createRenderEffect((_p$) => {
                var _v$ = range == null ? void 0 : range.location, _v$2 = ((range == null ? void 0 : range.location) || 0) + ((range == null ? void 0 : range.length) || 0), _v$3 = props.operator === "add" ? `var(${addContentHighlightBGName})` : `var(${delContentHighlightBGName})`;
                _v$ !== _p$.e && setAttribute(_el$7, "data-range-start", _p$.e = _v$);
                _v$2 !== _p$.t && setAttribute(_el$7, "data-range-end", _p$.t = _v$2);
                _v$3 !== _p$.a && ((_p$.a = _v$3) != null ? _el$8.style.setProperty("background-color", _v$3) : _el$8.style.removeProperty("background-color"));
                return _p$;
              }, {
                e: void 0,
                t: void 0,
                a: void 0
              });
              return _el$6;
            })();
          }();
        },
        get children() {
          var _el$ = _tmpl$$h(), _el$2 = _el$.firstChild;
          insert(_el$, (() => {
            var _c$ = createMemo(() => getIsNewLineSymbolChanged() === NewLineSymbol.NEWLINE);
            return () => _c$() && (() => {
              var _el$11 = _tmpl$5$6();
              `var(${diffFontSizeName})` != null ? _el$11.style.setProperty("width", `var(${diffFontSizeName})`) : _el$11.style.removeProperty("width");
              `var(${diffFontSizeName})` != null ? _el$11.style.setProperty("height", `var(${diffFontSizeName})`) : _el$11.style.removeProperty("height");
              insert(_el$11, createComponent(DiffNoNewLine, {}));
              createRenderEffect(() => className(_el$11, props.enableWrap ? "block !text-red-500" : "inline-block align-middle !text-red-500"));
              return _el$11;
            })();
          })(), null);
          createRenderEffect(() => {
            var _a;
            return _el$2.innerHTML = (_a = props.diffLine) == null ? void 0 : _a.plainTemplate;
          });
          return _el$;
        }
      });
    }
  });
};
const DiffSyntax = (props) => {
  const getRange = () => {
    var _a, _b;
    return (_b = (_a = props.diffLine) == null ? void 0 : _a.changes) == null ? void 0 : _b.range;
  };
  const getIsNewLineSymbolChanged = () => {
    var _a, _b;
    return (_b = (_a = props.diffLine) == null ? void 0 : _a.changes) == null ? void 0 : _b.newLineSymbol;
  };
  const initTemplateIfNeed = () => {
    var _a, _b;
    if (props.enableTemplate && props.diffLine && props.syntaxLine && props.operator && !((_a = props.diffLine) == null ? void 0 : _a.syntaxTemplate) && typeof getSyntaxDiffTemplate === "function") {
      getSyntaxDiffTemplate({
        diffLine: props.diffLine,
        syntaxLine: props.syntaxLine,
        operator: props.operator
      });
    }
    if (props.enableTemplate && props.syntaxLine && !((_b = props.syntaxLine) == null ? void 0 : _b.template)) {
      props.syntaxLine.template = getSyntaxLineTemplate(props.syntaxLine);
    }
  };
  createMemo(initTemplateIfNeed);
  return createComponent(Show, {
    get when() {
      return props.syntaxLine;
    },
    get fallback() {
      return createComponent(DiffString, {
        get rawLine() {
          return props.rawLine;
        },
        get diffLine() {
          return props.diffLine;
        },
        get operator() {
          return props.operator;
        },
        get enableWrap() {
          return props.enableWrap;
        },
        get enableTemplate() {
          return props.enableTemplate;
        }
      });
    },
    get children() {
      return createComponent(Show, {
        get when() {
          var _a, _b;
          return (_b = (_a = props.diffLine) == null ? void 0 : _a.changes) == null ? void 0 : _b.hasLineChange;
        },
        get fallback() {
          return createComponent(Show, {
            get when() {
              var _a;
              return props.enableTemplate && ((_a = props.syntaxLine) == null ? void 0 : _a.template);
            },
            get fallback() {
              return (() => {
                var _el$16 = _tmpl$7$4();
                insert(_el$16, createComponent(For, {
                  get each() {
                    var _a;
                    return ((_a = props.syntaxLine) == null ? void 0 : _a.nodeList) || [];
                  },
                  children: ({
                    node,
                    wrapper
                  }) => (() => {
                    var _el$17 = _tmpl$8$2();
                    insert(_el$17, () => node.value);
                    createRenderEffect((_p$) => {
                      var _a, _b, _c;
                      var _v$4 = node.startIndex, _v$5 = node.endIndex, _v$6 = (_b = (_a = wrapper == null ? void 0 : wrapper.properties) == null ? void 0 : _a.className) == null ? void 0 : _b.join(" "), _v$7 = (_c = wrapper == null ? void 0 : wrapper.properties) == null ? void 0 : _c.style;
                      _v$4 !== _p$.e && setAttribute(_el$17, "data-start", _p$.e = _v$4);
                      _v$5 !== _p$.t && setAttribute(_el$17, "data-end", _p$.t = _v$5);
                      _v$6 !== _p$.a && className(_el$17, _p$.a = _v$6);
                      _p$.o = style(_el$17, _v$7, _p$.o);
                      return _p$;
                    }, {
                      e: void 0,
                      t: void 0,
                      a: void 0,
                      o: void 0
                    });
                    return _el$17;
                  })()
                }));
                return _el$16;
              })();
            },
            get children() {
              var _el$14 = _tmpl$6$4(), _el$15 = _el$14.firstChild;
              createRenderEffect(() => {
                var _a;
                return _el$15.innerHTML = (_a = props.syntaxLine) == null ? void 0 : _a.template;
              });
              return _el$14;
            }
          });
        },
        get children() {
          return createComponent(Show, {
            get when() {
              var _a;
              return props.enableTemplate && ((_a = props.diffLine) == null ? void 0 : _a.syntaxTemplate);
            },
            get fallback() {
              return (() => {
                var _el$18 = _tmpl$9$1(), _el$19 = _el$18.firstChild;
                insert(_el$19, createComponent(For, {
                  get each() {
                    var _a;
                    return (_a = props.syntaxLine) == null ? void 0 : _a.nodeList;
                  },
                  children: ({
                    node,
                    wrapper
                  }) => {
                    if (node.endIndex < getRange().location || getRange().location + getRange().length < node.startIndex) {
                      return (() => {
                        var _el$20 = _tmpl$8$2();
                        insert(_el$20, () => node.value);
                        createRenderEffect((_p$) => {
                          var _a, _b, _c;
                          var _v$10 = node.startIndex, _v$11 = node.endIndex, _v$12 = (_b = (_a = wrapper == null ? void 0 : wrapper.properties) == null ? void 0 : _a.className) == null ? void 0 : _b.join(" "), _v$13 = (_c = wrapper == null ? void 0 : wrapper.properties) == null ? void 0 : _c.style;
                          _v$10 !== _p$.e && setAttribute(_el$20, "data-start", _p$.e = _v$10);
                          _v$11 !== _p$.t && setAttribute(_el$20, "data-end", _p$.t = _v$11);
                          _v$12 !== _p$.a && className(_el$20, _p$.a = _v$12);
                          _p$.o = style(_el$20, _v$13, _p$.o);
                          return _p$;
                        }, {
                          e: void 0,
                          t: void 0,
                          a: void 0,
                          o: void 0
                        });
                        return _el$20;
                      })();
                    } else {
                      const index1 = getRange().location - node.startIndex;
                      const index2 = index1 < 0 ? 0 : index1;
                      const str1 = node.value.slice(0, index2);
                      const str2 = node.value.slice(index2, index1 + getRange().length);
                      const str3 = node.value.slice(index1 + getRange().length);
                      const isStart = str1.length || getRange().location === node.startIndex;
                      const isEnd = str3.length || node.endIndex === getRange().location + getRange().length - 1;
                      const isLast = str2.includes("\n");
                      const _str2 = isLast ? str2.replace("\n", "").replace("\r", "") : str2;
                      return (() => {
                        var _el$21 = _tmpl$10$1(), _el$22 = _el$21.firstChild;
                        insert(_el$21, str1, _el$22);
                        (isStart ? "0.2em" : void 0) != null ? _el$22.style.setProperty("border-top-left-radius", isStart ? "0.2em" : void 0) : _el$22.style.removeProperty("border-top-left-radius");
                        (isStart ? "0.2em" : void 0) != null ? _el$22.style.setProperty("border-bottom-left-radius", isStart ? "0.2em" : void 0) : _el$22.style.removeProperty("border-bottom-left-radius");
                        (isEnd || isLast ? "0.2em" : void 0) != null ? _el$22.style.setProperty("border-top-right-radius", isEnd || isLast ? "0.2em" : void 0) : _el$22.style.removeProperty("border-top-right-radius");
                        (isEnd || isLast ? "0.2em" : void 0) != null ? _el$22.style.setProperty("border-bottom-right-radius", isEnd || isLast ? "0.2em" : void 0) : _el$22.style.removeProperty("border-bottom-right-radius");
                        insert(_el$22, isLast ? [_str2, (() => {
                          var _el$23 = _tmpl$4$7();
                          insert(_el$23, () => getSymbol(getIsNewLineSymbolChanged()));
                          return _el$23;
                        })()] : str2);
                        insert(_el$21, str3, null);
                        createRenderEffect((_p$) => {
                          var _a, _b, _c;
                          var _v$14 = node.startIndex, _v$15 = node.endIndex, _v$16 = (_b = (_a = wrapper == null ? void 0 : wrapper.properties) == null ? void 0 : _a.className) == null ? void 0 : _b.join(" "), _v$17 = (_c = wrapper == null ? void 0 : wrapper.properties) == null ? void 0 : _c.style, _v$18 = props.operator === "add" ? `var(${addContentHighlightBGName})` : `var(${delContentHighlightBGName})`;
                          _v$14 !== _p$.e && setAttribute(_el$21, "data-start", _p$.e = _v$14);
                          _v$15 !== _p$.t && setAttribute(_el$21, "data-end", _p$.t = _v$15);
                          _v$16 !== _p$.a && className(_el$21, _p$.a = _v$16);
                          _p$.o = style(_el$21, _v$17, _p$.o);
                          _v$18 !== _p$.i && ((_p$.i = _v$18) != null ? _el$22.style.setProperty("background-color", _v$18) : _el$22.style.removeProperty("background-color"));
                          return _p$;
                        }, {
                          e: void 0,
                          t: void 0,
                          a: void 0,
                          o: void 0,
                          i: void 0
                        });
                        return _el$21;
                      })();
                    }
                  }
                }));
                insert(_el$18, (() => {
                  var _c$4 = createMemo(() => getIsNewLineSymbolChanged() === NewLineSymbol.NEWLINE);
                  return () => _c$4() && (() => {
                    var _el$24 = _tmpl$5$6();
                    `var(${diffFontSizeName})` != null ? _el$24.style.setProperty("width", `var(${diffFontSizeName})`) : _el$24.style.removeProperty("width");
                    `var(${diffFontSizeName})` != null ? _el$24.style.setProperty("height", `var(${diffFontSizeName})`) : _el$24.style.removeProperty("height");
                    insert(_el$24, createComponent(DiffNoNewLine, {}));
                    createRenderEffect(() => className(_el$24, props.enableWrap ? "block !text-red-500" : "inline-block align-middle !text-red-500"));
                    return _el$24;
                  })();
                })(), null);
                createRenderEffect((_p$) => {
                  var _a, _b, _c;
                  var _v$8 = (_a = getRange()) == null ? void 0 : _a.location, _v$9 = (((_b = getRange()) == null ? void 0 : _b.location) || 0) + (((_c = getRange()) == null ? void 0 : _c.length) || 0);
                  _v$8 !== _p$.e && setAttribute(_el$19, "data-range-start", _p$.e = _v$8);
                  _v$9 !== _p$.t && setAttribute(_el$19, "data-range-end", _p$.t = _v$9);
                  return _p$;
                }, {
                  e: void 0,
                  t: void 0
                });
                return _el$18;
              })();
            },
            get children() {
              var _el$12 = _tmpl$6$4(), _el$13 = _el$12.firstChild;
              insert(_el$12, (() => {
                var _c$3 = createMemo(() => getIsNewLineSymbolChanged() === NewLineSymbol.NEWLINE);
                return () => _c$3() && (() => {
                  var _el$25 = _tmpl$5$6();
                  `var(${diffFontSizeName})` != null ? _el$25.style.setProperty("width", `var(${diffFontSizeName})`) : _el$25.style.removeProperty("width");
                  `var(${diffFontSizeName})` != null ? _el$25.style.setProperty("height", `var(${diffFontSizeName})`) : _el$25.style.removeProperty("height");
                  insert(_el$25, createComponent(DiffNoNewLine, {}));
                  createRenderEffect(() => className(_el$25, props.enableWrap ? "block !text-red-500" : "inline-block align-middle !text-red-500"));
                  return _el$25;
                })();
              })(), null);
              createRenderEffect(() => {
                var _a;
                return _el$13.innerHTML = (_a = props.diffLine) == null ? void 0 : _a.syntaxTemplate;
              });
              return _el$12;
            }
          });
        }
      });
    }
  });
};
const DiffContent = (props) => {
  const getIsAdded = () => {
    var _a;
    return ((_a = props.diffLine) == null ? void 0 : _a.type) === DiffLineType.Add;
  };
  const getIsDelete = () => {
    var _a;
    return ((_a = props.diffLine) == null ? void 0 : _a.type) === DiffLineType.Delete;
  };
  const getIsMaxLineLengthToIgnoreSyntax = () => {
    var _a, _b, _c;
    return ((_a = props.syntaxLine) == null ? void 0 : _a.nodeList) ? ((_c = (_b = props.syntaxLine) == null ? void 0 : _b.nodeList) == null ? void 0 : _c.length) > 150 : false;
  };
  const getIsEnableTemplate = () => {
    var _a, _b;
    return ((_b = (_a = props.diffFile).getIsEnableTemplate) == null ? void 0 : _b.call(_a)) ?? true;
  };
  return (() => {
    var _el$26 = _tmpl$11$1(), _el$27 = _el$26.firstChild;
    insert(_el$27, (() => {
      var _c$5 = createMemo(() => !!getIsAdded());
      return () => _c$5() ? "+" : getIsDelete() ? "-" : " ";
    })());
    insert(_el$26, (() => {
      var _c$6 = createMemo(() => !!(props.enableHighlight && props.syntaxLine && !getIsMaxLineLengthToIgnoreSyntax()));
      return () => _c$6() ? createComponent(DiffSyntax, {
        get operator() {
          return createMemo(() => !!getIsAdded())() ? "add" : getIsDelete() ? "del" : void 0;
        },
        get rawLine() {
          return props.rawLine;
        },
        get diffLine() {
          return props.diffLine;
        },
        get syntaxLine() {
          return props.syntaxLine;
        },
        get enableWrap() {
          return props.enableWrap;
        },
        get enableTemplate() {
          return getIsEnableTemplate();
        }
      }) : createComponent(DiffString, {
        get operator() {
          return createMemo(() => !!getIsAdded())() ? "add" : getIsDelete() ? "del" : void 0;
        },
        get rawLine() {
          return props.rawLine;
        },
        get diffLine() {
          return props.diffLine;
        },
        get plainLine() {
          return props.plainLine;
        },
        get enableWrap() {
          return props.enableWrap;
        },
        get enableTemplate() {
          return getIsEnableTemplate();
        }
      });
    })(), null);
    createRenderEffect((_p$) => {
      var _v$19 = props.enableWrap ? "pre-wrap" : "pre", _v$20 = props.enableWrap ? "break-all" : "initial", _v$21 = getIsAdded() ? "+" : getIsDelete() ? "-" : void 0;
      _v$19 !== _p$.e && ((_p$.e = _v$19) != null ? _el$26.style.setProperty("white-space", _v$19) : _el$26.style.removeProperty("white-space"));
      _v$20 !== _p$.t && ((_p$.t = _v$20) != null ? _el$26.style.setProperty("word-break", _v$20) : _el$26.style.removeProperty("word-break"));
      _v$21 !== _p$.a && setAttribute(_el$27, "data-operator", _p$.a = _v$21);
      return _p$;
    }, {
      e: void 0,
      t: void 0,
      a: void 0
    });
    return _el$26;
  })();
};
const DiffWidgetContext = createContext();
const useDiffWidgetContext = () => useContext(DiffWidgetContext);
var _tmpl$$g = /* @__PURE__ */ template(`<tr>`), _tmpl$2$c = /* @__PURE__ */ template(`<td><span>`), _tmpl$3$9 = /* @__PURE__ */ template(`<td>`), _tmpl$4$6 = /* @__PURE__ */ template(`<td colspan=2><span>&ensp;`);
const DiffSplitContentLine$1 = (props) => {
  const [_, setWidget] = useDiffWidgetContext() || [];
  const enableAddWidget = useEnableAddWidget();
  const enableHighlight = useEnableHighlight();
  const onAddWidgetClick = useOnAddWidgetClick();
  const getCurrentLine = () => props.side === SplitSide.old ? props.diffFile.getSplitLeftLine(props.index) : props.diffFile.getSplitRightLine(props.index);
  const currentLine = createMemo(getCurrentLine);
  const getCUrrentLineHasDiff = () => {
    var _a;
    return !!((_a = currentLine()) == null ? void 0 : _a.diff);
  };
  const currentLineHasDiff = createMemo(getCUrrentLineHasDiff);
  const currentLineHasChange = createMemo(() => {
    var _a;
    return checkDiffLineIncludeChange((_a = currentLine()) == null ? void 0 : _a.diff);
  });
  const getCurrentLineHasHidden = () => {
    var _a;
    return (_a = currentLine()) == null ? void 0 : _a.isHidden;
  };
  const currentLineHasHidden = createMemo(getCurrentLineHasHidden);
  const currentLineHasContent = createMemo(() => {
    var _a;
    return !!((_a = currentLine()) == null ? void 0 : _a.lineNumber);
  });
  const getCurrentSyntaxLine = () => {
    var _a, _b;
    return props.side === SplitSide.old ? props.diffFile.getOldSyntaxLine(((_a = currentLine()) == null ? void 0 : _a.lineNumber) || 0) : props.diffFile.getNewSyntaxLine(((_b = currentLine()) == null ? void 0 : _b.lineNumber) || 0);
  };
  const getCurrentPlainLine = () => {
    var _a, _b;
    return props.side === SplitSide.old ? props.diffFile.getOldPlainLine(((_a = currentLine()) == null ? void 0 : _a.lineNumber) || 0) : props.diffFile.getNewPlainLine(((_b = currentLine()) == null ? void 0 : _b.lineNumber) || 0);
  };
  const [currentSyntaxLine, setCurrentSyntaxLine] = createSignal(getCurrentSyntaxLine());
  const [currentPlainLine, setCurrentPlainLine] = createSignal(getCurrentPlainLine());
  const init = () => {
    setCurrentSyntaxLine(getCurrentSyntaxLine);
    setCurrentPlainLine(getCurrentPlainLine);
  };
  createEffect(() => {
    init();
    const cb = props.diffFile.subscribe(init);
    onCleanup(cb);
  });
  const onOpenAddWidget = (lineNumber, side) => setWidget == null ? void 0 : setWidget({
    side,
    lineNumber
  });
  const isAdded = () => {
    var _a, _b;
    return ((_b = (_a = currentLine()) == null ? void 0 : _a.diff) == null ? void 0 : _b.type) === DiffLineType.Add;
  };
  const isDelete = () => {
    var _a, _b;
    return ((_b = (_a = currentLine()) == null ? void 0 : _a.diff) == null ? void 0 : _b.type) === DiffLineType.Delete;
  };
  return createComponent(Show, {
    get when() {
      return !currentLineHasHidden();
    },
    get children() {
      var _el$ = _tmpl$$g();
      insert(_el$, (() => {
        var _c$ = createMemo(() => !!currentLineHasContent());
        return () => _c$() ? [(() => {
          var _el$2 = _tmpl$2$c(), _el$3 = _el$2.firstChild;
          `var(${diffAsideWidthName})` != null ? _el$2.style.setProperty("width", `var(${diffAsideWidthName})`) : _el$2.style.removeProperty("width");
          `var(${diffAsideWidthName})` != null ? _el$2.style.setProperty("min-width", `var(${diffAsideWidthName})`) : _el$2.style.removeProperty("min-width");
          `var(${diffAsideWidthName})` != null ? _el$2.style.setProperty("max-width", `var(${diffAsideWidthName})`) : _el$2.style.removeProperty("max-width");
          insert(_el$2, (() => {
            var _c$2 = createMemo(() => !!(currentLineHasDiff() && enableAddWidget()));
            return () => _c$2() && createComponent(DiffSplitAddWidget, {
              get index() {
                return props.index;
              },
              get lineNumber() {
                var _a;
                return ((_a = currentLine()) == null ? void 0 : _a.lineNumber) || 0;
              },
              get side() {
                return props.side;
              },
              get diffFile() {
                return props.diffFile;
              },
              onWidgetClick: onAddWidgetClick,
              "class": "absolute left-[100%] z-[1] translate-x-[-50%]",
              onOpenAddWidget
            });
          })(), _el$3);
          insert(_el$3, () => {
            var _a;
            return (_a = currentLine()) == null ? void 0 : _a.lineNumber;
          });
          createRenderEffect((_p$) => {
            var _a;
            var _v$5 = `diff-line-${SplitSide[props.side]}-num sticky left-0 z-[1] w-[1%] min-w-[40px] select-none pl-[10px] pr-[10px] text-right align-top`, _v$6 = getLineNumberBG(isAdded(), isDelete(), currentLineHasDiff()), _v$7 = `var(${currentLineHasDiff() ? plainLineNumberColorName : expandLineNumberColorName})`, _v$8 = (_a = currentLine()) == null ? void 0 : _a.lineNumber, _v$9 = currentLineHasChange() ? void 0 : 0.5;
            _v$5 !== _p$.e && className(_el$2, _p$.e = _v$5);
            _v$6 !== _p$.t && ((_p$.t = _v$6) != null ? _el$2.style.setProperty("background-color", _v$6) : _el$2.style.removeProperty("background-color"));
            _v$7 !== _p$.a && ((_p$.a = _v$7) != null ? _el$2.style.setProperty("color", _v$7) : _el$2.style.removeProperty("color"));
            _v$8 !== _p$.o && setAttribute(_el$3, "data-line-num", _p$.o = _v$8);
            _v$9 !== _p$.i && ((_p$.i = _v$9) != null ? _el$3.style.setProperty("opacity", _v$9) : _el$3.style.removeProperty("opacity"));
            return _p$;
          }, {
            e: void 0,
            t: void 0,
            a: void 0,
            o: void 0,
            i: void 0
          });
          return _el$2;
        })(), (() => {
          var _el$4 = _tmpl$3$9();
          insert(_el$4, createComponent(DiffContent, {
            enableWrap: false,
            get diffFile() {
              return props.diffFile;
            },
            get rawLine() {
              var _a;
              return ((_a = currentLine()) == null ? void 0 : _a.value) || "";
            },
            get diffLine() {
              var _a;
              return (_a = currentLine()) == null ? void 0 : _a.diff;
            },
            get plainLine() {
              return currentPlainLine();
            },
            get syntaxLine() {
              return currentSyntaxLine();
            },
            get enableHighlight() {
              return !!enableHighlight();
            }
          }));
          createRenderEffect((_p$) => {
            var _v$10 = `diff-line-${SplitSide[props.side]}-content pr-[10px] align-top`, _v$11 = getContentBG(isAdded(), isDelete(), currentLineHasDiff());
            _v$10 !== _p$.e && className(_el$4, _p$.e = _v$10);
            _v$11 !== _p$.t && ((_p$.t = _v$11) != null ? _el$4.style.setProperty("background-color", _v$11) : _el$4.style.removeProperty("background-color"));
            return _p$;
          }, {
            e: void 0,
            t: void 0
          });
          return _el$4;
        })()] : (() => {
          var _el$5 = _tmpl$4$6();
          `var(${emptyBGName})` != null ? _el$5.style.setProperty("background-color", `var(${emptyBGName})`) : _el$5.style.removeProperty("background-color");
          createRenderEffect(() => className(_el$5, `diff-line-${SplitSide[props.side]}-placeholder select-none`));
          return _el$5;
        })();
      })());
      createRenderEffect((_p$) => {
        var _v$ = props.lineNumber, _v$2 = currentLineHasDiff() || !currentLineHasContent() ? "diff" : "plain", _v$3 = SplitSide[props.side], _v$4 = "diff-line" + (currentLineHasContent() ? " group" : "");
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && setAttribute(_el$, "data-state", _p$.t = _v$2);
        _v$3 !== _p$.a && setAttribute(_el$, "data-side", _p$.a = _v$3);
        _v$4 !== _p$.o && className(_el$, _p$.o = _v$4);
        return _p$;
      }, {
        e: void 0,
        t: void 0,
        a: void 0,
        o: void 0
      });
      return _el$;
    }
  });
};
var _tmpl$$f = /* @__PURE__ */ template(`<tr data-state=extend class="diff-line diff-line-extend">`), _tmpl$2$b = /* @__PURE__ */ template(`<td colspan=2><div class="diff-line-extend-wrapper sticky left-0 z-[1]">`), _tmpl$3$8 = /* @__PURE__ */ template(`<td colspan=2><div>`);
const DiffSplitExtendLine$1 = (props) => {
  const extendData = useExtendData();
  const renderExtend = useRenderExtend();
  const lineSelector = createMemo(() => `div[data-line="${props.lineNumber}-extend-content"]`);
  const lineWrapperSelector = createMemo(() => `tr[data-line="${props.lineNumber}-extend"]`);
  const wrapperSelector = createMemo(() => props.side === SplitSide.old ? ".old-diff-table-wrapper" : ".new-diff-table-wrapper");
  const oldLine = createMemo(() => props.diffFile.getSplitLeftLine(props.index));
  const newLine = createMemo(() => props.diffFile.getSplitRightLine(props.index));
  const enableExpand = createMemo(() => props.diffFile.getExpandEnabled());
  const oldLineExtend = createMemo(() => {
    var _a, _b, _c;
    return (_c = (_a = extendData()) == null ? void 0 : _a.oldFile) == null ? void 0 : _c[((_b = oldLine()) == null ? void 0 : _b.lineNumber) || ""];
  });
  const newLineExtend = createMemo(() => {
    var _a, _b, _c;
    return (_c = (_a = extendData()) == null ? void 0 : _a.newFile) == null ? void 0 : _c[((_b = newLine()) == null ? void 0 : _b.lineNumber) || ""];
  });
  const currentItem = createMemo(() => props.side === SplitSide.old ? oldLine() : newLine());
  const currentIsHidden = createMemo(() => {
    var _a;
    return (_a = currentItem()) == null ? void 0 : _a.isHidden;
  });
  const currentExtend = createMemo(() => props.side === SplitSide.old ? oldLineExtend() : newLineExtend());
  const currentLineNumber = createMemo(() => {
    var _a, _b;
    return props.side === SplitSide.old ? (_a = oldLine()) == null ? void 0 : _a.lineNumber : (_b = newLine()) == null ? void 0 : _b.lineNumber;
  });
  const currentIsShow = createMemo(() => Boolean((oldLineExtend() || newLineExtend()) && (!currentIsHidden() || enableExpand()) && renderExtend()));
  const currentEnable = createMemo(() => (props.side === SplitSide.old ? !!oldLineExtend() : !!newLineExtend()) && currentIsShow());
  const extendSide = createMemo(() => SplitSide[currentExtend() ? props.side : props.side === SplitSide.new ? SplitSide.old : SplitSide.new]);
  useSyncHeight({
    selector: lineSelector,
    wrapper: lineWrapperSelector,
    side: extendSide,
    enable: currentIsShow
  });
  const width = useDomWidth({
    selector: wrapperSelector,
    enable: currentEnable
  });
  return createComponent(Show, {
    get when() {
      return currentIsShow();
    },
    get children() {
      var _el$ = _tmpl$$f();
      insert(_el$, (() => {
        var _c$ = createMemo(() => !!(renderExtend() && currentExtend()));
        return () => _c$() ? (() => {
          var _el$2 = _tmpl$2$b(), _el$3 = _el$2.firstChild;
          insert(_el$3, (() => {
            var _c$2 = createMemo(() => width() > 0);
            return () => {
              var _a, _b;
              return _c$2() && ((_b = renderExtend()) == null ? void 0 : _b({
                diffFile: props.diffFile,
                side: props.side,
                lineNumber: currentLineNumber() || 0,
                data: (_a = currentExtend()) == null ? void 0 : _a.data,
                onUpdate: props.diffFile.notifyAll
              }));
            };
          })());
          createRenderEffect((_p$) => {
            var _v$3 = `diff-line-extend-${SplitSide[props.side]}-content p-0`, _v$4 = `${props.lineNumber}-extend-content`, _v$5 = SplitSide[props.side], _v$6 = width() + "px";
            _v$3 !== _p$.e && className(_el$2, _p$.e = _v$3);
            _v$4 !== _p$.t && setAttribute(_el$3, "data-line", _p$.t = _v$4);
            _v$5 !== _p$.a && setAttribute(_el$3, "data-side", _p$.a = _v$5);
            _v$6 !== _p$.o && ((_p$.o = _v$6) != null ? _el$3.style.setProperty("width", _v$6) : _el$3.style.removeProperty("width"));
            return _p$;
          }, {
            e: void 0,
            t: void 0,
            a: void 0,
            o: void 0
          });
          return _el$2;
        })() : (() => {
          var _el$4 = _tmpl$3$8(), _el$5 = _el$4.firstChild;
          `var(${emptyBGName})` != null ? _el$4.style.setProperty("background-color", `var(${emptyBGName})`) : _el$4.style.removeProperty("background-color");
          createRenderEffect((_p$) => {
            var _v$7 = `diff-line-extend-${SplitSide[props.side]}-placeholder select-none p-0`, _v$8 = `${props.lineNumber}-extend-content`, _v$9 = SplitSide[props.side];
            _v$7 !== _p$.e && className(_el$4, _p$.e = _v$7);
            _v$8 !== _p$.t && setAttribute(_el$5, "data-line", _p$.t = _v$8);
            _v$9 !== _p$.a && setAttribute(_el$5, "data-side", _p$.a = _v$9);
            return _p$;
          }, {
            e: void 0,
            t: void 0,
            a: void 0
          });
          return _el$4;
        })();
      })());
      createRenderEffect((_p$) => {
        var _v$ = `${props.lineNumber}-extend`, _v$2 = SplitSide[props.side];
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && setAttribute(_el$, "data-side", _p$.t = _v$2);
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$;
    }
  });
};
var _tmpl$$e = /* @__PURE__ */ template(`<svg aria-hidden=true height=16 viewBox="0 0 16 16"version=1.1 width=16><path d="m8.177 14.323 2.896-2.896a.25.25 0 0 0-.177-.427H8.75V7.764a.75.75 0 1 0-1.5 0V11H5.104a.25.25 0 0 0-.177.427l2.896 2.896a.25.25 0 0 0 .354 0ZM2.25 5a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM6 4.25a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5a.75.75 0 0 1 .75.75ZM8.25 5a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM12 4.25a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5a.75.75 0 0 1 .75.75Zm2.25.75a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5Z">`), _tmpl$2$a = /* @__PURE__ */ template(`<svg aria-hidden=true height=16 viewBox="0 0 16 16"version=1.1 width=16><path d="M7.823 1.677 4.927 4.573A.25.25 0 0 0 5.104 5H7.25v3.236a.75.75 0 1 0 1.5 0V5h2.146a.25.25 0 0 0 .177-.427L8.177 1.677a.25.25 0 0 0-.354 0ZM13.75 11a.75.75 0 0 0 0 1.5h.5a.75.75 0 0 0 0-1.5h-.5Zm-3.75.75a.75.75 0 0 1 .75-.75h.5a.75.75 0 0 1 0 1.5h-.5a.75.75 0 0 1-.75-.75ZM7.75 11a.75.75 0 0 0 0 1.5h.5a.75.75 0 0 0 0-1.5h-.5ZM4 11.75a.75.75 0 0 1 .75-.75h.5a.75.75 0 0 1 0 1.5h-.5a.75.75 0 0 1-.75-.75ZM1.75 11a.75.75 0 0 0 0 1.5h.5a.75.75 0 0 0 0-1.5h-.5Z">`), _tmpl$3$7 = /* @__PURE__ */ template(`<svg aria-hidden=true height=16 viewBox="0 0 16 16"version=1.1 width=16><path d="m8.177.677 2.896 2.896a.25.25 0 0 1-.177.427H8.75v1.25a.75.75 0 0 1-1.5 0V4H5.104a.25.25 0 0 1-.177-.427L7.823.677a.25.25 0 0 1 .354 0ZM7.25 10.75a.75.75 0 0 1 1.5 0V12h2.146a.25.25 0 0 1 .177.427l-2.896 2.896a.25.25 0 0 1-.354 0l-2.896-2.896A.25.25 0 0 1 5.104 12H7.25v-1.25Zm-5-2a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM6 8a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5A.75.75 0 0 1 6 8Zm2.25.75a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM12 8a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5A.75.75 0 0 1 12 8Zm2.25.75a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5Z">`);
const ExpandDown = (props) => {
  return (() => {
    var _el$ = _tmpl$$e();
    createRenderEffect(() => setAttribute(_el$, "class", props.class));
    return _el$;
  })();
};
const ExpandUp = (props) => {
  return (() => {
    var _el$2 = _tmpl$2$a();
    createRenderEffect(() => setAttribute(_el$2, "class", props.class));
    return _el$2;
  })();
};
const ExpandAll = (props) => {
  return (() => {
    var _el$3 = _tmpl$3$7();
    createRenderEffect(() => setAttribute(_el$3, "class", props.class));
    return _el$3;
  })();
};
var _tmpl$$d = /* @__PURE__ */ template(`<tr data-state=hunk class="diff-line diff-line-hunk">`), _tmpl$2$9 = /* @__PURE__ */ template(`<td class="diff-line-hunk-action sticky left-0 w-[1%] min-w-[40px] select-none p-[1px]">`), _tmpl$3$6 = /* @__PURE__ */ template(`<td class="diff-line-hunk-content pr-[10px] align-middle"><div class=pl-[1.5em]>`), _tmpl$4$5 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand Up"data-title="Expand Up">`), _tmpl$5$5 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand Down"data-title="Expand Down">`), _tmpl$6$3 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand All"data-title="Expand All">`), _tmpl$7$3 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[2px]"title="Expand Down"data-title="Expand Down">`), _tmpl$8$1 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[2px]"title="Expand Up"data-title="Expand Up">`), _tmpl$9 = /* @__PURE__ */ template(`<div class=min-h-[28px]>&ensp;`), _tmpl$10 = /* @__PURE__ */ template(`<td class="diff-line-hunk-placeholder select-none"colspan=2><div class=min-h-[28px]>&ensp;`), _tmpl$11 = /* @__PURE__ */ template(`<tr data-state=hunk class="diff-line diff-line-hunk"><td class="diff-line-hunk-action sticky left-0 w-[1%] min-w-[40px] select-none p-[1px]"></td><td class="diff-line-hunk-content pr-[10px] align-middle"><div class=pl-[1.5em]>`);
const DiffSplitHunkLineGitHub$1 = (props) => {
  const currentHunk = createMemo(() => props.diffFile.getSplitHunkLine(props.index));
  const enableExpand = createMemo(() => props.diffFile.getExpandEnabled());
  const couldExpand = createMemo(() => {
    var _a;
    return enableExpand() && ((_a = currentHunk()) == null ? void 0 : _a.splitInfo);
  });
  const lineSelector = createMemo(() => `tr[data-line="${props.lineNumber}-hunk"]`);
  const currentShowExpand = createMemo(() => props.side === SplitSide.old);
  const checkCurrentShowExpandAll = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.endHiddenIndex - hunk.splitInfo.startHiddenIndex < composeLen;
  };
  const [currentShowExpandAll, setCurrentShowExpandAll] = createSignal(checkCurrentShowExpandAll());
  const checkCurrentIsShow = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.startHiddenIndex < hunk.splitInfo.endHiddenIndex;
  };
  const [currentIsShow, setCurrentIsShow] = createSignal(checkCurrentIsShow());
  const currentIsFirstLine = createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isFirst;
  });
  const currentIsPureHunk = createMemo(() => {
    const hunk = currentHunk();
    return hunk && props.diffFile._getIsPureDiffRender() && !hunk.splitInfo;
  });
  const currentIsLastLine = createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isLast;
  });
  createEffect(() => {
    const init = () => {
      setCurrentIsShow(checkCurrentIsShow);
      setCurrentShowExpandAll(checkCurrentShowExpandAll);
    };
    init();
    const cb = props.diffFile.subscribe(init);
    onCleanup(cb);
  });
  const currentSyncHeightSide = createMemo(() => SplitSide[SplitSide.old]);
  const currentEnableSyncHeight = createMemo(() => props.side === SplitSide.new && !!currentIsShow());
  useSyncHeight({
    selector: lineSelector,
    wrapper: lineSelector,
    side: currentSyncHeightSide,
    enable: currentEnableSyncHeight
  });
  return createComponent(Show, {
    get when() {
      return currentIsShow() || currentIsPureHunk();
    },
    get children() {
      var _el$ = _tmpl$$d();
      `var(${hunkContentBGName})` != null ? _el$.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$.style.removeProperty("background-color");
      insert(_el$, (() => {
        var _c$ = createMemo(() => !!currentShowExpand());
        return () => _c$() ? [(() => {
          var _el$2 = _tmpl$2$9();
          `var(${hunkLineNumberBGName})` != null ? _el$2.style.setProperty("background-color", `var(${hunkLineNumberBGName})`) : _el$2.style.removeProperty("background-color");
          `var(${plainLineNumberColorName})` != null ? _el$2.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$2.style.removeProperty("color");
          `var(${diffAsideWidthName})` != null ? _el$2.style.setProperty("width", `var(${diffAsideWidthName})`) : _el$2.style.removeProperty("width");
          `var(${diffAsideWidthName})` != null ? _el$2.style.setProperty("min-width", `var(${diffAsideWidthName})`) : _el$2.style.removeProperty("min-width");
          `var(${diffAsideWidthName})` != null ? _el$2.style.setProperty("max-width", `var(${diffAsideWidthName})`) : _el$2.style.removeProperty("max-width");
          insert(_el$2, (() => {
            var _c$2 = createMemo(() => !!couldExpand());
            return () => _c$2() ? createMemo(() => !!currentIsFirstLine())() ? (() => {
              var _el$5 = _tmpl$4$5();
              _el$5.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
              insert(_el$5, createComponent(ExpandUp, {
                "class": "fill-current"
              }));
              return _el$5;
            })() : createMemo(() => !!currentIsLastLine())() ? (() => {
              var _el$6 = _tmpl$5$5();
              _el$6.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
              insert(_el$6, createComponent(ExpandDown, {
                "class": "fill-current"
              }));
              return _el$6;
            })() : createMemo(() => !!currentShowExpandAll())() ? (() => {
              var _el$7 = _tmpl$6$3();
              _el$7.$$click = () => props.diffFile.onSplitHunkExpand("all", props.index);
              insert(_el$7, createComponent(ExpandAll, {
                "class": "fill-current"
              }));
              return _el$7;
            })() : [(() => {
              var _el$8 = _tmpl$7$3();
              _el$8.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
              insert(_el$8, createComponent(ExpandDown, {
                "class": "fill-current"
              }));
              return _el$8;
            })(), (() => {
              var _el$9 = _tmpl$8$1();
              _el$9.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
              insert(_el$9, createComponent(ExpandUp, {
                "class": "fill-current"
              }));
              return _el$9;
            })()] : _tmpl$9();
          })());
          return _el$2;
        })(), (() => {
          var _el$3 = _tmpl$3$6(), _el$4 = _el$3.firstChild;
          `var(${hunkContentBGName})` != null ? _el$3.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$3.style.removeProperty("background-color");
          `var(${hunkContentColorName})` != null ? _el$4.style.setProperty("color", `var(${hunkContentColorName})`) : _el$4.style.removeProperty("color");
          insert(_el$4, () => {
            var _a, _b, _c;
            return ((_b = (_a = currentHunk()) == null ? void 0 : _a.splitInfo) == null ? void 0 : _b.plainText) || ((_c = currentHunk()) == null ? void 0 : _c.text);
          });
          return _el$3;
        })()] : (() => {
          var _el$11 = _tmpl$10();
          `var(${hunkContentBGName})` != null ? _el$11.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$11.style.removeProperty("background-color");
          return _el$11;
        })();
      })());
      createRenderEffect((_p$) => {
        var _v$ = `${props.lineNumber}-hunk`, _v$2 = SplitSide[props.side];
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && setAttribute(_el$, "data-side", _p$.t = _v$2);
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$;
    }
  });
};
const DiffSplitHunkLineGitLab$1 = (props) => {
  const currentHunk = createMemo(() => props.diffFile.getSplitHunkLine(props.index));
  const enableExpand = createMemo(() => props.diffFile.getExpandEnabled());
  const couldExpand = createMemo(() => {
    var _a;
    return enableExpand() && ((_a = currentHunk()) == null ? void 0 : _a.splitInfo);
  });
  const lineSelector = createMemo(() => `tr[data-line="${props.lineNumber}-hunk"]`);
  const checkCurrentShowExpandAll = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.endHiddenIndex - hunk.splitInfo.startHiddenIndex < composeLen;
  };
  const [currentShowExpandAll, setCurrentShowExpandAll] = createSignal(checkCurrentShowExpandAll());
  const checkCurrentIsShow = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.startHiddenIndex < hunk.splitInfo.endHiddenIndex;
  };
  const [currentIsShow, setCurrentIsShow] = createSignal(checkCurrentIsShow());
  const currentIsFirstLine = createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isFirst;
  });
  const currentIsPureHunk = createMemo(() => {
    const hunk = currentHunk();
    return hunk && props.diffFile._getIsPureDiffRender() && !hunk.splitInfo;
  });
  const currentIsLastLine = createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isLast;
  });
  const currentSyncHeightSide = createMemo(() => SplitSide[SplitSide.old]);
  const currentEnableSyncHeight = createMemo(() => props.side === SplitSide.new && !!currentIsShow());
  createEffect(() => {
    const init = () => {
      setCurrentIsShow(checkCurrentIsShow);
      setCurrentShowExpandAll(checkCurrentShowExpandAll);
    };
    init();
    const cb = props.diffFile.subscribe(init);
    onCleanup(cb);
  });
  useSyncHeight({
    selector: lineSelector,
    wrapper: lineSelector,
    side: currentSyncHeightSide,
    enable: currentEnableSyncHeight
  });
  return createComponent(Show, {
    get when() {
      return currentIsShow() || currentIsPureHunk();
    },
    get children() {
      var _el$12 = _tmpl$11(), _el$13 = _el$12.firstChild, _el$14 = _el$13.nextSibling, _el$15 = _el$14.firstChild;
      `var(${hunkContentBGName})` != null ? _el$12.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$12.style.removeProperty("background-color");
      `var(${hunkLineNumberBGName})` != null ? _el$13.style.setProperty("background-color", `var(${hunkLineNumberBGName})`) : _el$13.style.removeProperty("background-color");
      `var(${plainLineNumberColorName})` != null ? _el$13.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$13.style.removeProperty("color");
      `var(${diffAsideWidthName})` != null ? _el$13.style.setProperty("width", `var(${diffAsideWidthName})`) : _el$13.style.removeProperty("width");
      `var(${diffAsideWidthName})` != null ? _el$13.style.setProperty("min-width", `var(${diffAsideWidthName})`) : _el$13.style.removeProperty("min-width");
      `var(${diffAsideWidthName})` != null ? _el$13.style.setProperty("max-width", `var(${diffAsideWidthName})`) : _el$13.style.removeProperty("max-width");
      insert(_el$13, (() => {
        var _c$3 = createMemo(() => !!couldExpand());
        return () => _c$3() ? createMemo(() => !!currentIsFirstLine())() ? (() => {
          var _el$16 = _tmpl$4$5();
          _el$16.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$16, createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$16;
        })() : createMemo(() => !!currentIsLastLine())() ? (() => {
          var _el$17 = _tmpl$5$5();
          _el$17.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$17, createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$17;
        })() : createMemo(() => !!currentShowExpandAll())() ? (() => {
          var _el$18 = _tmpl$6$3();
          _el$18.$$click = () => props.diffFile.onSplitHunkExpand("all", props.index);
          insert(_el$18, createComponent(ExpandAll, {
            "class": "fill-current"
          }));
          return _el$18;
        })() : [(() => {
          var _el$19 = _tmpl$7$3();
          _el$19.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$19, createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$19;
        })(), (() => {
          var _el$20 = _tmpl$8$1();
          _el$20.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$20, createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$20;
        })()] : _tmpl$9();
      })());
      `var(${hunkContentBGName})` != null ? _el$14.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$14.style.removeProperty("background-color");
      `var(${hunkContentColorName})` != null ? _el$15.style.setProperty("color", `var(${hunkContentColorName})`) : _el$15.style.removeProperty("color");
      insert(_el$15, () => {
        var _a, _b, _c;
        return ((_b = (_a = currentHunk()) == null ? void 0 : _a.splitInfo) == null ? void 0 : _b.plainText) || ((_c = currentHunk()) == null ? void 0 : _c.text);
      });
      createRenderEffect((_p$) => {
        var _v$3 = `${props.lineNumber}-hunk`, _v$4 = SplitSide[props.side];
        _v$3 !== _p$.e && setAttribute(_el$12, "data-line", _p$.e = _v$3);
        _v$4 !== _p$.t && setAttribute(_el$12, "data-side", _p$.t = _v$4);
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$12;
    }
  });
};
const DiffSplitHunkLine$1 = (props) => {
  const mode = useMode();
  return createComponent(Show, {
    get when() {
      return mode() === DiffModeEnum.SplitGitHub || mode() === DiffModeEnum.Split;
    },
    get fallback() {
      return createComponent(DiffSplitHunkLineGitLab$1, {
        get index() {
          return props.index;
        },
        get side() {
          return props.side;
        },
        get diffFile() {
          return props.diffFile;
        },
        get lineNumber() {
          return props.lineNumber;
        }
      });
    },
    get children() {
      return createComponent(DiffSplitHunkLineGitHub$1, {
        get index() {
          return props.index;
        },
        get side() {
          return props.side;
        },
        get diffFile() {
          return props.diffFile;
        },
        get lineNumber() {
          return props.lineNumber;
        }
      });
    }
  });
};
delegateEvents(["click"]);
var _tmpl$$c = /* @__PURE__ */ template(`<tr data-state=widget class="diff-line diff-line-widget">`), _tmpl$2$8 = /* @__PURE__ */ template(`<td colspan=2><div class="diff-line-widget-wrapper sticky left-0 z-[1]">`), _tmpl$3$5 = /* @__PURE__ */ template(`<td colspan=2><div>`);
const DiffSplitWidgetLine$1 = (props) => {
  const renderWidget = useRenderWidget();
  const [widget, setWidget] = useDiffWidgetContext() || [];
  const oldLine = createMemo(() => props.diffFile.getSplitLeftLine(props.index));
  const newLine = createMemo(() => props.diffFile.getSplitRightLine(props.index));
  const oldLineWidget = createMemo(() => {
    var _a, _b, _c, _d;
    return !!((_a = oldLine()) == null ? void 0 : _a.lineNumber) && ((_b = widget == null ? void 0 : widget()) == null ? void 0 : _b.side) === SplitSide.old && ((_c = widget == null ? void 0 : widget()) == null ? void 0 : _c.lineNumber) === ((_d = oldLine()) == null ? void 0 : _d.lineNumber);
  });
  const newLineWidget = createMemo(() => {
    var _a, _b, _c, _d;
    return !!((_a = newLine()) == null ? void 0 : _a.lineNumber) && ((_b = widget == null ? void 0 : widget()) == null ? void 0 : _b.side) === SplitSide.new && ((_c = widget == null ? void 0 : widget()) == null ? void 0 : _c.lineNumber) === ((_d = newLine()) == null ? void 0 : _d.lineNumber);
  });
  const currentLine = createMemo(() => props.side === SplitSide.old ? oldLine() : newLine());
  const currentIsHidden = createMemo(() => {
    var _a;
    return (_a = currentLine()) == null ? void 0 : _a.isHidden;
  });
  const lineSelector = createMemo(() => `div[data-line="${props.lineNumber}-widget-content"]`);
  const lineWrapperSelector = createMemo(() => `tr[data-line="${props.lineNumber}-widget"]`);
  const wrapperSelector = createMemo(() => props.side === SplitSide.old ? ".old-diff-table-wrapper" : ".new-diff-table-wrapper");
  const currentWidget = createMemo(() => props.side === SplitSide.old ? oldLineWidget() : newLineWidget());
  const observeSide = createMemo(() => SplitSide[currentWidget() ? props.side : props.side === SplitSide.old ? SplitSide.new : SplitSide.old]);
  const currentIsShow = createMemo(() => (!!oldLineWidget() || !!newLineWidget()) && !currentIsHidden() && !!renderWidget);
  const currentEnable = createMemo(() => currentWidget() && !!currentIsShow());
  const onCloseWidget = () => setWidget == null ? void 0 : setWidget({});
  useSyncHeight({
    selector: lineSelector,
    wrapper: lineWrapperSelector,
    side: observeSide,
    enable: currentIsShow
  });
  const width = useDomWidth({
    selector: wrapperSelector,
    enable: currentEnable
  });
  return createComponent(Show, {
    get when() {
      return currentIsShow();
    },
    get children() {
      var _el$ = _tmpl$$c();
      insert(_el$, (() => {
        var _c$ = createMemo(() => !!currentWidget());
        return () => _c$() ? (() => {
          var _el$2 = _tmpl$2$8(), _el$3 = _el$2.firstChild;
          insert(_el$3, (() => {
            var _c$2 = createMemo(() => width() > 0);
            return () => {
              var _a, _b;
              return _c$2() && ((_b = renderWidget()) == null ? void 0 : _b({
                diffFile: props.diffFile,
                side: props.side,
                lineNumber: ((_a = currentLine()) == null ? void 0 : _a.lineNumber) || 0,
                onClose: onCloseWidget
              }));
            };
          })());
          createRenderEffect((_p$) => {
            var _v$3 = `diff-line-widget-${SplitSide[props.side]}-content p-0`, _v$4 = `${props.lineNumber}-widget-content`, _v$5 = SplitSide[props.side], _v$6 = width() + "px";
            _v$3 !== _p$.e && className(_el$2, _p$.e = _v$3);
            _v$4 !== _p$.t && setAttribute(_el$3, "data-line", _p$.t = _v$4);
            _v$5 !== _p$.a && setAttribute(_el$3, "data-side", _p$.a = _v$5);
            _v$6 !== _p$.o && ((_p$.o = _v$6) != null ? _el$3.style.setProperty("width", _v$6) : _el$3.style.removeProperty("width"));
            return _p$;
          }, {
            e: void 0,
            t: void 0,
            a: void 0,
            o: void 0
          });
          return _el$2;
        })() : (() => {
          var _el$4 = _tmpl$3$5(), _el$5 = _el$4.firstChild;
          `var(${emptyBGName})` != null ? _el$4.style.setProperty("background-color", `var(${emptyBGName})`) : _el$4.style.removeProperty("background-color");
          createRenderEffect((_p$) => {
            var _v$7 = `diff-line-widget-${SplitSide[props.side]}-placeholder select-none p-0`, _v$8 = `${props.lineNumber}-widget-content`, _v$9 = SplitSide[props.side];
            _v$7 !== _p$.e && className(_el$4, _p$.e = _v$7);
            _v$8 !== _p$.t && setAttribute(_el$5, "data-line", _p$.t = _v$8);
            _v$9 !== _p$.a && setAttribute(_el$5, "data-side", _p$.a = _v$9);
            return _p$;
          }, {
            e: void 0,
            t: void 0,
            a: void 0
          });
          return _el$4;
        })();
      })());
      createRenderEffect((_p$) => {
        var _v$ = `${props.lineNumber}-widget`, _v$2 = SplitSide[props.side];
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && setAttribute(_el$, "data-side", _p$.t = _v$2);
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$;
    }
  });
};
var _tmpl$$b = /* @__PURE__ */ template(`<table><colgroup><col><col></colgroup><thead class=hidden><tr><th scope=col> line number</th><th scope=col> line content</th></tr></thead><tbody class="diff-table-body leading-[1.4]">`), _tmpl$2$7 = /* @__PURE__ */ template(`<div class="split-diff-view split-diff-view-normal flex w-full basis-[50%]"><style data-select-style></style><div class="old-diff-table-wrapper diff-table-scroll-container w-full overflow-x-auto overflow-y-hidden"></div><div class="diff-split-line w-[1.5px]"></div><div class="new-diff-table-wrapper diff-table-scroll-container w-full overflow-x-auto overflow-y-hidden">`);
const DiffSplitViewTable = (props) => {
  const className$1 = createMemo(() => props.side === SplitSide.new ? "new-diff-table" : "old-diff-table");
  const getAllLines = () => getSplitContentLines(props.diffFile);
  const [lines, setLines] = createSignal(getAllLines());
  const selectState = props.selectState;
  createEffect(() => {
    const init = () => setLines(getAllLines);
    init();
    const cb = props.diffFile.subscribe(init);
    onCleanup(cb);
  });
  const onMouseDown = (e) => {
    var _a, _b;
    let ele = e.target;
    if (ele && (ele == null ? void 0 : ele.nodeName) === "BUTTON") {
      removeAllSelection();
      return;
    }
    while (ele && ele instanceof HTMLElement) {
      const state = ele.getAttribute("data-state");
      if (state) {
        if (state === "extend" || state === "hunk" || state === "widget") {
          if (selectState.current !== void 0) {
            selectState.current = void 0;
            (_a = props.onSelect) == null ? void 0 : _a.call(props, void 0);
            removeAllSelection();
          }
        } else {
          if (selectState.current !== props.side) {
            selectState.current = props.side;
            (_b = props.onSelect) == null ? void 0 : _b.call(props, props.side);
            removeAllSelection();
          }
        }
        return;
      }
      ele = ele.parentElement;
    }
  };
  return (() => {
    var _el$ = _tmpl$$b(), _el$2 = _el$.firstChild, _el$3 = _el$2.firstChild, _el$4 = _el$3.nextSibling, _el$5 = _el$2.nextSibling, _el$6 = _el$5.firstChild, _el$7 = _el$6.firstChild, _el$8 = _el$7.firstChild, _el$9 = _el$7.nextSibling, _el$10 = _el$9.firstChild, _el$11 = _el$5.nextSibling;
    insert(_el$7, () => SplitSide[props.side], _el$8);
    insert(_el$9, () => SplitSide[props.side], _el$10);
    _el$11.$$mousedown = onMouseDown;
    insert(_el$11, createComponent(For, {
      get each() {
        return lines();
      },
      children: (item) => [createComponent(DiffSplitHunkLine$1, {
        get index() {
          return item.index;
        },
        get side() {
          return props.side;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), createComponent(DiffSplitContentLine$1, {
        get index() {
          return item.index;
        },
        get side() {
          return props.side;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), createComponent(DiffSplitWidgetLine$1, {
        get index() {
          return item.index;
        },
        get side() {
          return props.side;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), createComponent(DiffSplitExtendLine$1, {
        get index() {
          return item.index;
        },
        get side() {
          return props.side;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      })]
    }), null);
    insert(_el$11, createComponent(DiffSplitHunkLine$1, {
      get side() {
        return props.side;
      },
      get index() {
        return props.diffFile.splitLineLength;
      },
      get lineNumber() {
        return props.diffFile.splitLineLength;
      },
      get diffFile() {
        return props.diffFile;
      }
    }), null);
    createRenderEffect((_p$) => {
      var _v$ = `${className$1()} w-full border-collapse border-spacing-0`, _v$2 = SplitSide[props.side], _v$3 = `diff-table-${SplitSide[props.side]}-num-col`, _v$4 = `diff-table-${SplitSide[props.side]}-content-col`;
      _v$ !== _p$.e && className(_el$, _p$.e = _v$);
      _v$2 !== _p$.t && setAttribute(_el$, "data-mode", _p$.t = _v$2);
      _v$3 !== _p$.a && className(_el$3, _p$.a = _v$3);
      _v$4 !== _p$.o && className(_el$4, _p$.o = _v$4);
      return _p$;
    }, {
      e: void 0,
      t: void 0,
      a: void 0,
      o: void 0
    });
    return _el$;
  })();
};
const DiffSplitViewNormal = (props) => {
  const isMounted = useIsMounted();
  const [ref1, setRef1] = createSignal(null);
  const [ref2, setRef2] = createSignal(null);
  const [styleRef, setStyleRef] = createSignal(null);
  const maxText = createMemo(() => Math.max(props.diffFile.fileLineLength, props.diffFile.splitLineLength).toString());
  const initSyncScroll = () => {
    if (!isMounted()) return;
    const left = ref1();
    const right = ref2();
    if (!left || !right) return;
    const clean = syncScroll(left, right);
    onCleanup(clean);
  };
  createEffect(initSyncScroll);
  const selectState = {
    current: void 0
  };
  const onSelect = (side) => {
    const ele = styleRef();
    if (!ele) return;
    if (!side) {
      ele.textContent = "";
    } else {
      ele.textContent = `#${getId()} [data-state="extend"] {user-select: none} 
#${getId()} [data-state="hunk"] {user-select: none} 
#${getId()} [data-state="widget"] {user-select: none}`;
    }
  };
  const fontSize = useFontSize();
  const font = createMemo(() => ({
    fontSize: `${fontSize() || 14}px`,
    fontFamily: "Menlo, Consolas, monospace"
  }));
  const width = useTextWidth({
    text: maxText,
    font
  });
  const computedWidth = createMemo(() => Math.max(40, width() + 25));
  const getId = () => `diff-split-view-${props.diffFile.getId()}`;
  return (() => {
    var _el$12 = _tmpl$2$7(), _el$13 = _el$12.firstChild, _el$14 = _el$13.nextSibling, _el$15 = _el$14.nextSibling, _el$16 = _el$15.nextSibling;
    use(setStyleRef, _el$13);
    use(setRef1, _el$14);
    _el$14.style.setProperty("overscroll-behavior-x", "none");
    _el$14.style.setProperty("font-family", "Menlo, Consolas, monospace");
    `var(${diffFontSizeName})` != null ? _el$14.style.setProperty("font-size", `var(${diffFontSizeName})`) : _el$14.style.removeProperty("font-size");
    insert(_el$14, createComponent(DiffSplitViewTable, {
      get side() {
        return SplitSide.old;
      },
      get diffFile() {
        return props.diffFile;
      },
      onSelect,
      selectState
    }));
    `var(${borderColorName})` != null ? _el$15.style.setProperty("background-color", `var(${borderColorName})`) : _el$15.style.removeProperty("background-color");
    use(setRef2, _el$16);
    _el$16.style.setProperty("overscroll-behavior-x", "none");
    _el$16.style.setProperty("font-family", "Menlo, Consolas, monospace");
    `var(${diffFontSizeName})` != null ? _el$16.style.setProperty("font-size", `var(${diffFontSizeName})`) : _el$16.style.removeProperty("font-size");
    insert(_el$16, createComponent(DiffSplitViewTable, {
      get side() {
        return SplitSide.new;
      },
      get diffFile() {
        return props.diffFile;
      },
      onSelect,
      selectState
    }));
    createRenderEffect((_p$) => {
      var _v$5 = {
        [diffAsideWidthName]: `${Math.round(computedWidth())}px`
      }, _v$6 = {
        [diffAsideWidthName]: `${Math.round(computedWidth())}px`
      };
      _p$.e = style(_el$14, _v$5, _p$.e);
      _p$.t = style(_el$16, _v$6, _p$.t);
      return _p$;
    }, {
      e: void 0,
      t: void 0
    });
    return _el$12;
  })();
};
delegateEvents(["mousedown"]);
var _tmpl$$a = /* @__PURE__ */ template(`<tr class=diff-line>`), _tmpl$2$6 = /* @__PURE__ */ template(`<td class="diff-line-old-num group relative w-[1%] min-w-[40px] select-none pl-[10px] pr-[10px] text-right align-top"><span>`), _tmpl$3$4 = /* @__PURE__ */ template(`<td class="diff-line-old-content group relative pr-[10px] align-top">`), _tmpl$4$4 = /* @__PURE__ */ template(`<td class="diff-line-old-placeholder select-none"colspan=2><span>&ensp;`), _tmpl$5$4 = /* @__PURE__ */ template(`<td class="diff-line-new-num group relative w-[1%] min-w-[40px] select-none border-l-[1px] pl-[10px] pr-[10px] text-right align-top"><span>`), _tmpl$6$2 = /* @__PURE__ */ template(`<td class="diff-line-new-content group relative pr-[10px] align-top">`), _tmpl$7$2 = /* @__PURE__ */ template(`<td class="diff-line-new-placeholder select-none border-l-[1px]"colspan=2><span>&ensp;`);
const DiffSplitContentLine = (props) => {
  const [_, setWidget] = useDiffWidgetContext() || [];
  const enableAddWidget = useEnableAddWidget();
  const enableHighlight = useEnableHighlight();
  const onAddWidgetClick = useOnAddWidgetClick();
  const oldLine = createMemo(() => props.diffFile.getSplitLeftLine(props.index));
  const newLine = createMemo(() => props.diffFile.getSplitRightLine(props.index));
  const getOldSyntaxLine = () => {
    var _a;
    return props.diffFile.getOldSyntaxLine(((_a = oldLine()) == null ? void 0 : _a.lineNumber) || 0);
  };
  const getNewSyntaxLine = () => {
    var _a;
    return props.diffFile.getNewSyntaxLine(((_a = newLine()) == null ? void 0 : _a.lineNumber) || 0);
  };
  const [oldSyntaxLine, setOldSyntaxLine] = createSignal(getOldSyntaxLine());
  const [newSyntaxLine, setNewSyntaxLine] = createSignal(getNewSyntaxLine());
  const getOldPlainLine = () => {
    var _a;
    return props.diffFile.getOldPlainLine(((_a = oldLine()) == null ? void 0 : _a.lineNumber) || 0);
  };
  const getNewPlainLine = () => {
    var _a;
    return props.diffFile.getNewPlainLine(((_a = newLine()) == null ? void 0 : _a.lineNumber) || 0);
  };
  const [oldPlainLine, setOldPlainLine] = createSignal(getOldPlainLine());
  const [newPlainLine, setNewPlainLine] = createSignal(getNewPlainLine());
  const checkHasDiff = () => {
    var _a, _b;
    return !!((_a = oldLine()) == null ? void 0 : _a.diff) || !!((_b = newLine()) == null ? void 0 : _b.diff);
  };
  const hasDiff = createMemo(checkHasDiff);
  const checkHasChange = () => {
    var _a, _b;
    return checkDiffLineIncludeChange((_a = oldLine()) == null ? void 0 : _a.diff) || checkDiffLineIncludeChange((_b = newLine()) == null ? void 0 : _b.diff);
  };
  const hasChange = createMemo(checkHasChange);
  const checkHasHidden = () => {
    var _a, _b;
    return ((_a = oldLine()) == null ? void 0 : _a.isHidden) && ((_b = newLine()) == null ? void 0 : _b.isHidden);
  };
  const hasHidden = createMemo(checkHasHidden);
  const oldLineIsDelete = () => {
    var _a, _b;
    return ((_b = (_a = oldLine()) == null ? void 0 : _a.diff) == null ? void 0 : _b.type) === DiffLineType.Delete;
  };
  const newLineIsAdded = () => {
    var _a, _b;
    return ((_b = (_a = newLine()) == null ? void 0 : _a.diff) == null ? void 0 : _b.type) === DiffLineType.Add;
  };
  createEffect(() => {
    const init = () => {
      setOldSyntaxLine(getOldSyntaxLine);
      setNewSyntaxLine(getNewSyntaxLine);
      setOldPlainLine(getOldPlainLine);
      setNewPlainLine(getNewPlainLine);
    };
    init();
    const cb = props.diffFile.subscribe(init);
    onCleanup(cb);
  });
  const onOpenAddWidget = (lineNumber, side) => setWidget == null ? void 0 : setWidget({
    side,
    lineNumber
  });
  return createComponent(Show, {
    get when() {
      return !hasHidden();
    },
    get children() {
      var _el$ = _tmpl$$a();
      insert(_el$, (() => {
        var _c$ = createMemo(() => {
          var _a;
          return !!((_a = oldLine()) == null ? void 0 : _a.lineNumber);
        });
        return () => _c$() ? [(() => {
          var _el$2 = _tmpl$2$6(), _el$3 = _el$2.firstChild;
          insert(_el$2, (() => {
            var _c$3 = createMemo(() => !!(hasDiff() && enableAddWidget()));
            return () => _c$3() && createComponent(DiffSplitAddWidget, {
              get index() {
                return props.index;
              },
              get lineNumber() {
                var _a;
                return ((_a = oldLine()) == null ? void 0 : _a.lineNumber) || 0;
              },
              get side() {
                return SplitSide.old;
              },
              get diffFile() {
                return props.diffFile;
              },
              onWidgetClick: onAddWidgetClick,
              "class": "absolute left-[100%] z-[1] translate-x-[-50%]",
              onOpenAddWidget
            });
          })(), _el$3);
          insert(_el$3, () => {
            var _a;
            return (_a = oldLine()) == null ? void 0 : _a.lineNumber;
          });
          createRenderEffect((_p$) => {
            var _a;
            var _v$3 = getLineNumberBG(false, oldLineIsDelete(), hasDiff()), _v$4 = `var(${hasDiff() ? plainLineNumberColorName : expandLineNumberColorName})`, _v$5 = (_a = oldLine()) == null ? void 0 : _a.lineNumber, _v$6 = hasChange() ? void 0 : 0.5;
            _v$3 !== _p$.e && ((_p$.e = _v$3) != null ? _el$2.style.setProperty("background-color", _v$3) : _el$2.style.removeProperty("background-color"));
            _v$4 !== _p$.t && ((_p$.t = _v$4) != null ? _el$2.style.setProperty("color", _v$4) : _el$2.style.removeProperty("color"));
            _v$5 !== _p$.a && setAttribute(_el$3, "data-line-num", _p$.a = _v$5);
            _v$6 !== _p$.o && ((_p$.o = _v$6) != null ? _el$3.style.setProperty("opacity", _v$6) : _el$3.style.removeProperty("opacity"));
            return _p$;
          }, {
            e: void 0,
            t: void 0,
            a: void 0,
            o: void 0
          });
          return _el$2;
        })(), (() => {
          var _el$4 = _tmpl$3$4();
          insert(_el$4, (() => {
            var _c$4 = createMemo(() => !!(hasDiff() && enableAddWidget()));
            return () => _c$4() && createComponent(DiffSplitAddWidget, {
              get index() {
                return props.index;
              },
              get lineNumber() {
                var _a;
                return ((_a = oldLine()) == null ? void 0 : _a.lineNumber) || 0;
              },
              get side() {
                return SplitSide.old;
              },
              get diffFile() {
                return props.diffFile;
              },
              onWidgetClick: onAddWidgetClick,
              "class": "absolute right-[100%] z-[1] translate-x-[50%]",
              onOpenAddWidget
            });
          })(), null);
          insert(_el$4, createComponent(DiffContent, {
            enableWrap: true,
            get diffFile() {
              return props.diffFile;
            },
            get rawLine() {
              var _a;
              return ((_a = oldLine()) == null ? void 0 : _a.value) || "";
            },
            get diffLine() {
              var _a;
              return (_a = oldLine()) == null ? void 0 : _a.diff;
            },
            get plainLine() {
              return oldPlainLine();
            },
            get syntaxLine() {
              return oldSyntaxLine();
            },
            get enableHighlight() {
              return !!enableHighlight();
            }
          }), null);
          createRenderEffect((_p$) => {
            var _v$7 = getContentBG(false, oldLineIsDelete(), hasDiff()), _v$8 = SplitSide[SplitSide.old];
            _v$7 !== _p$.e && ((_p$.e = _v$7) != null ? _el$4.style.setProperty("background-color", _v$7) : _el$4.style.removeProperty("background-color"));
            _v$8 !== _p$.t && setAttribute(_el$4, "data-side", _p$.t = _v$8);
            return _p$;
          }, {
            e: void 0,
            t: void 0
          });
          return _el$4;
        })()] : (() => {
          var _el$5 = _tmpl$4$4();
          `var(${emptyBGName})` != null ? _el$5.style.setProperty("background-color", `var(${emptyBGName})`) : _el$5.style.removeProperty("background-color");
          return _el$5;
        })();
      })(), null);
      insert(_el$, (() => {
        var _c$2 = createMemo(() => {
          var _a;
          return !!((_a = newLine()) == null ? void 0 : _a.lineNumber);
        });
        return () => _c$2() ? [(() => {
          var _el$6 = _tmpl$5$4(), _el$7 = _el$6.firstChild;
          `var(${borderColorName})` != null ? _el$6.style.setProperty("border-left-color", `var(${borderColorName})`) : _el$6.style.removeProperty("border-left-color");
          _el$6.style.setProperty("border-left-style", "solid");
          insert(_el$6, (() => {
            var _c$5 = createMemo(() => !!(hasDiff() && enableAddWidget()));
            return () => _c$5() && createComponent(DiffSplitAddWidget, {
              get index() {
                return props.index;
              },
              get lineNumber() {
                var _a;
                return ((_a = newLine()) == null ? void 0 : _a.lineNumber) || 0;
              },
              get side() {
                return SplitSide.new;
              },
              get diffFile() {
                return props.diffFile;
              },
              onWidgetClick: onAddWidgetClick,
              "class": "absolute left-[100%] z-[1] translate-x-[-50%]",
              onOpenAddWidget
            });
          })(), _el$7);
          insert(_el$7, () => {
            var _a;
            return (_a = newLine()) == null ? void 0 : _a.lineNumber;
          });
          createRenderEffect((_p$) => {
            var _a;
            var _v$9 = getLineNumberBG(newLineIsAdded(), false, hasDiff()), _v$10 = `var(${hasDiff() ? plainLineNumberColorName : expandLineNumberColorName})`, _v$11 = (_a = newLine()) == null ? void 0 : _a.lineNumber, _v$12 = hasChange() ? void 0 : 0.5;
            _v$9 !== _p$.e && ((_p$.e = _v$9) != null ? _el$6.style.setProperty("background-color", _v$9) : _el$6.style.removeProperty("background-color"));
            _v$10 !== _p$.t && ((_p$.t = _v$10) != null ? _el$6.style.setProperty("color", _v$10) : _el$6.style.removeProperty("color"));
            _v$11 !== _p$.a && setAttribute(_el$7, "data-line-num", _p$.a = _v$11);
            _v$12 !== _p$.o && ((_p$.o = _v$12) != null ? _el$7.style.setProperty("opacity", _v$12) : _el$7.style.removeProperty("opacity"));
            return _p$;
          }, {
            e: void 0,
            t: void 0,
            a: void 0,
            o: void 0
          });
          return _el$6;
        })(), (() => {
          var _el$8 = _tmpl$6$2();
          insert(_el$8, (() => {
            var _c$6 = createMemo(() => !!(hasDiff() && enableAddWidget()));
            return () => _c$6() && createComponent(DiffSplitAddWidget, {
              get index() {
                return props.index;
              },
              get lineNumber() {
                var _a;
                return ((_a = newLine()) == null ? void 0 : _a.lineNumber) || 0;
              },
              get side() {
                return SplitSide.new;
              },
              get diffFile() {
                return props.diffFile;
              },
              onWidgetClick: onAddWidgetClick,
              "class": "absolute right-[100%] z-[1] translate-x-[50%]",
              onOpenAddWidget
            });
          })(), null);
          insert(_el$8, createComponent(DiffContent, {
            enableWrap: true,
            get diffFile() {
              return props.diffFile;
            },
            get rawLine() {
              var _a;
              return ((_a = newLine()) == null ? void 0 : _a.value) || "";
            },
            get diffLine() {
              var _a;
              return (_a = newLine()) == null ? void 0 : _a.diff;
            },
            get plainLine() {
              return newPlainLine();
            },
            get syntaxLine() {
              return newSyntaxLine();
            },
            get enableHighlight() {
              return !!enableHighlight();
            }
          }), null);
          createRenderEffect((_p$) => {
            var _v$13 = getContentBG(newLineIsAdded(), false, hasDiff()), _v$14 = SplitSide[SplitSide.new];
            _v$13 !== _p$.e && ((_p$.e = _v$13) != null ? _el$8.style.setProperty("background-color", _v$13) : _el$8.style.removeProperty("background-color"));
            _v$14 !== _p$.t && setAttribute(_el$8, "data-side", _p$.t = _v$14);
            return _p$;
          }, {
            e: void 0,
            t: void 0
          });
          return _el$8;
        })()] : (() => {
          var _el$9 = _tmpl$7$2();
          `var(${emptyBGName})` != null ? _el$9.style.setProperty("background-color", `var(${emptyBGName})`) : _el$9.style.removeProperty("background-color");
          `var(${borderColorName})` != null ? _el$9.style.setProperty("border-left-color", `var(${borderColorName})`) : _el$9.style.removeProperty("border-left-color");
          _el$9.style.setProperty("border-left-style", "solid");
          return _el$9;
        })();
      })(), null);
      createRenderEffect((_p$) => {
        var _v$ = props.lineNumber, _v$2 = hasDiff() ? "diff" : "plain";
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && setAttribute(_el$, "data-state", _p$.t = _v$2);
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$;
    }
  });
};
var _tmpl$$9 = /* @__PURE__ */ template(`<tr data-state=extend class="diff-line diff-line-extend">`), _tmpl$2$5 = /* @__PURE__ */ template(`<td class="diff-line-extend-old-content p-0"colspan=2><div class=diff-line-extend-wrapper>`), _tmpl$3$3 = /* @__PURE__ */ template(`<td class="diff-line-extend-old-placeholder select-none p-0"colspan=2>`), _tmpl$4$3 = /* @__PURE__ */ template(`<td class="diff-line-extend-new-content border-l-[1px] p-0"colspan=2><div class=diff-line-extend-wrapper>`), _tmpl$5$3 = /* @__PURE__ */ template(`<td class="diff-line-extend-new-placeholder select-none border-l-[1px] p-0"colspan=2>`);
const DiffSplitExtendLine = (props) => {
  const extendData = useExtendData();
  const renderExtend = useRenderExtend();
  const oldLine = createMemo(() => props.diffFile.getSplitLeftLine(props.index));
  const newLine = createMemo(() => props.diffFile.getSplitRightLine(props.index));
  const enableExpand = createMemo(() => props.diffFile.getExpandEnabled());
  const oldLineExtend = createMemo(() => {
    var _a, _b, _c;
    return (_c = (_a = extendData()) == null ? void 0 : _a.oldFile) == null ? void 0 : _c[((_b = oldLine()) == null ? void 0 : _b.lineNumber) || ""];
  });
  const newLineExtend = createMemo(() => {
    var _a, _b, _c;
    return (_c = (_a = extendData()) == null ? void 0 : _a.newFile) == null ? void 0 : _c[((_b = newLine()) == null ? void 0 : _b.lineNumber) || ""];
  });
  const checkIsShow = () => {
    const oldExtend = oldLineExtend();
    const newExtend = newLineExtend();
    const oldLineData = oldLine();
    const newLineData = newLine();
    const enableExpandValue = enableExpand();
    const renderExtendValue = renderExtend();
    return (oldExtend || newExtend) && (!(oldLineData == null ? void 0 : oldLineData.isHidden) && !(newLineData == null ? void 0 : newLineData.isHidden) || enableExpandValue) && renderExtendValue;
  };
  const currentIsShow = createMemo(checkIsShow);
  return createComponent(Show, {
    get when() {
      return currentIsShow();
    },
    get children() {
      var _el$ = _tmpl$$9();
      insert(_el$, (() => {
        var _c$ = createMemo(() => !!(renderExtend() && oldLineExtend()));
        return () => _c$() ? (() => {
          var _el$2 = _tmpl$2$5(), _el$3 = _el$2.firstChild;
          insert(_el$3, () => {
            var _a, _b, _c;
            return (_c = renderExtend()) == null ? void 0 : _c({
              diffFile: props.diffFile,
              side: SplitSide.old,
              lineNumber: ((_a = oldLine()) == null ? void 0 : _a.lineNumber) || 0,
              data: (_b = oldLineExtend()) == null ? void 0 : _b.data,
              onUpdate: props.diffFile.notifyAll
            });
          });
          return _el$2;
        })() : (() => {
          var _el$4 = _tmpl$3$3();
          `var(${emptyBGName})` != null ? _el$4.style.setProperty("background-color", `var(${emptyBGName})`) : _el$4.style.removeProperty("background-color");
          return _el$4;
        })();
      })(), null);
      insert(_el$, (() => {
        var _c$2 = createMemo(() => !!(renderExtend() && newLineExtend()));
        return () => _c$2() ? (() => {
          var _el$5 = _tmpl$4$3(), _el$6 = _el$5.firstChild;
          `var(${borderColorName})` != null ? _el$5.style.setProperty("border-left-color", `var(${borderColorName})`) : _el$5.style.removeProperty("border-left-color");
          _el$5.style.setProperty("border-left-style", "solid");
          insert(_el$6, () => {
            var _a, _b, _c;
            return (_c = renderExtend()) == null ? void 0 : _c({
              diffFile: props.diffFile,
              side: SplitSide.new,
              lineNumber: ((_a = newLine()) == null ? void 0 : _a.lineNumber) || 0,
              data: (_b = newLineExtend()) == null ? void 0 : _b.data,
              onUpdate: props.diffFile.notifyAll
            });
          });
          return _el$5;
        })() : (() => {
          var _el$7 = _tmpl$5$3();
          `var(${emptyBGName})` != null ? _el$7.style.setProperty("background-color", `var(${emptyBGName})`) : _el$7.style.removeProperty("background-color");
          `var(${borderColorName})` != null ? _el$7.style.setProperty("border-left-color", `var(${borderColorName})`) : _el$7.style.removeProperty("border-left-color");
          _el$7.style.setProperty("border-left-style", "solid");
          return _el$7;
        })();
      })(), null);
      createRenderEffect(() => setAttribute(_el$, "data-line", `${props.lineNumber}-extend`));
      return _el$;
    }
  });
};
var _tmpl$$8 = /* @__PURE__ */ template(`<tr data-state=hunk class="diff-line diff-line-hunk"><td class="diff-line-hunk-action relative w-[1%] min-w-[40px] select-none p-[1px]"></td><td class="diff-line-hunk-content pr-[10px] align-middle"colspan=3><div class=pl-[1.5em]>`), _tmpl$2$4 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand Up"data-title="Expand Up">`), _tmpl$3$2 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand Down"data-title="Expand Down">`), _tmpl$4$2 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand All"data-title="Expand All">`), _tmpl$5$2 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[2px]"title="Expand Down"data-title="Expand Down">`), _tmpl$6$1 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[2px]"title="Expand Up"data-title="Expand Up">`), _tmpl$7$1 = /* @__PURE__ */ template(`<div class=min-h-[28px]>&ensp;`), _tmpl$8 = /* @__PURE__ */ template(`<tr data-state=hunk class="diff-line diff-line-hunk"><td class="diff-line-hunk-action relative w-[1%] min-w-[40px] select-none p-[1px]"></td><td class="diff-line-hunk-content pr-[10px] align-middle"><div class=pl-[1.5em]></div></td><td class="diff-line-hunk-action relative z-[1] w-[1%] min-w-[40px] select-none border-l-[1px] p-[1px]"></td><td class="diff-line-hunk-content relative pr-[10px] align-middle"><div class=pl-[1.5em]>`);
const DiffSplitHunkLineGitHub = (props) => {
  const currentHunk = createMemo(() => props.diffFile.getSplitHunkLine(props.index));
  const enableExpand = createMemo(() => props.diffFile.getExpandEnabled());
  const couldExpand = createMemo(() => {
    var _a;
    return enableExpand() && ((_a = currentHunk()) == null ? void 0 : _a.splitInfo);
  });
  const checkCurrentShowExpandAll = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.endHiddenIndex - hunk.splitInfo.startHiddenIndex < composeLen;
  };
  const [currentShowExpandAll, setCurrentShowExpandAll] = createSignal(checkCurrentShowExpandAll());
  const checkCurrentIsShow = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.startHiddenIndex < hunk.splitInfo.endHiddenIndex;
  };
  const [currentIsShow, setCurrentIsShow] = createSignal(checkCurrentIsShow());
  const currentIsFirstLine = createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isFirst;
  });
  const currentIsPureHunk = createMemo(() => {
    const hunk = currentHunk();
    return hunk && props.diffFile._getIsPureDiffRender() && !hunk.splitInfo;
  });
  const currentIsLastLine = createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isLast;
  });
  createEffect(() => {
    const init = () => {
      setCurrentIsShow(checkCurrentIsShow);
      setCurrentShowExpandAll(checkCurrentShowExpandAll);
    };
    init();
    const cb = props.diffFile.subscribe(init);
    onCleanup(cb);
  });
  return createComponent(Show, {
    get when() {
      return currentIsShow() || currentIsPureHunk();
    },
    get children() {
      var _el$ = _tmpl$$8(), _el$2 = _el$.firstChild, _el$3 = _el$2.nextSibling, _el$4 = _el$3.firstChild;
      `var(${hunkLineNumberBGName})` != null ? _el$2.style.setProperty("background-color", `var(${hunkLineNumberBGName})`) : _el$2.style.removeProperty("background-color");
      `var(${plainLineNumberColorName})` != null ? _el$2.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$2.style.removeProperty("color");
      insert(_el$2, (() => {
        var _c$ = createMemo(() => !!couldExpand());
        return () => _c$() ? createMemo(() => !!currentIsFirstLine())() ? (() => {
          var _el$5 = _tmpl$2$4();
          _el$5.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$5, createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$5;
        })() : createMemo(() => !!currentIsLastLine())() ? (() => {
          var _el$6 = _tmpl$3$2();
          _el$6.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$6, createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$6;
        })() : createMemo(() => !!currentShowExpandAll())() ? (() => {
          var _el$7 = _tmpl$4$2();
          _el$7.$$click = () => props.diffFile.onSplitHunkExpand("all", props.index);
          insert(_el$7, createComponent(ExpandAll, {
            "class": "fill-current"
          }));
          return _el$7;
        })() : [(() => {
          var _el$8 = _tmpl$5$2();
          _el$8.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$8, createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$8;
        })(), (() => {
          var _el$9 = _tmpl$6$1();
          _el$9.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$9, createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$9;
        })()] : _tmpl$7$1();
      })());
      `var(${hunkContentBGName})` != null ? _el$3.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$3.style.removeProperty("background-color");
      `var(${hunkContentColorName})` != null ? _el$4.style.setProperty("color", `var(${hunkContentColorName})`) : _el$4.style.removeProperty("color");
      insert(_el$4, () => {
        var _a, _b, _c;
        return ((_b = (_a = currentHunk()) == null ? void 0 : _a.splitInfo) == null ? void 0 : _b.plainText) || ((_c = currentHunk()) == null ? void 0 : _c.text);
      });
      createRenderEffect(() => setAttribute(_el$, "data-line", `${props.lineNumber}-hunk`));
      return _el$;
    }
  });
};
const DiffSplitHunkLineGitLab = (props) => {
  const currentHunk = createMemo(() => props.diffFile.getSplitHunkLine(props.index));
  const enableExpand = createMemo(() => props.diffFile.getExpandEnabled());
  const couldExpand = createMemo(() => {
    var _a;
    return enableExpand() && ((_a = currentHunk()) == null ? void 0 : _a.splitInfo);
  });
  const checkCurrentShowExpandAll = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.endHiddenIndex - hunk.splitInfo.startHiddenIndex < composeLen;
  };
  const [currentShowExpandAll, setCurrentShowExpandAll] = createSignal(checkCurrentShowExpandAll());
  const checkCurrentIsShow = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.startHiddenIndex < hunk.splitInfo.endHiddenIndex;
  };
  const [currentIsShow, setCurrentIsShow] = createSignal(checkCurrentIsShow());
  const currentIsFirstLine = createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isFirst;
  });
  const currentIsPureHunk = createMemo(() => {
    const hunk = currentHunk();
    return hunk && props.diffFile._getIsPureDiffRender() && !hunk.splitInfo;
  });
  const currentIsLastLine = createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isLast;
  });
  createEffect(() => {
    const init = () => {
      setCurrentShowExpandAll(checkCurrentShowExpandAll());
      setCurrentIsShow(checkCurrentIsShow());
    };
    init();
    const cb = props.diffFile.subscribe(init);
    onCleanup(cb);
  });
  return createComponent(Show, {
    get when() {
      return currentIsShow() || currentIsPureHunk();
    },
    get children() {
      var _el$11 = _tmpl$8(), _el$12 = _el$11.firstChild, _el$13 = _el$12.nextSibling, _el$14 = _el$13.firstChild, _el$15 = _el$13.nextSibling, _el$16 = _el$15.nextSibling, _el$17 = _el$16.firstChild;
      `var(${hunkLineNumberBGName})` != null ? _el$12.style.setProperty("background-color", `var(${hunkLineNumberBGName})`) : _el$12.style.removeProperty("background-color");
      `var(${plainLineNumberColorName})` != null ? _el$12.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$12.style.removeProperty("color");
      insert(_el$12, (() => {
        var _c$2 = createMemo(() => !!couldExpand());
        return () => _c$2() ? createMemo(() => !!currentIsFirstLine())() ? (() => {
          var _el$18 = _tmpl$2$4();
          _el$18.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$18, createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$18;
        })() : createMemo(() => !!currentIsLastLine())() ? (() => {
          var _el$19 = _tmpl$3$2();
          _el$19.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$19, createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$19;
        })() : createMemo(() => !!currentShowExpandAll())() ? (() => {
          var _el$20 = _tmpl$4$2();
          _el$20.$$click = () => props.diffFile.onSplitHunkExpand("all", props.index);
          insert(_el$20, createComponent(ExpandAll, {
            "class": "fill-current"
          }));
          return _el$20;
        })() : [(() => {
          var _el$21 = _tmpl$5$2();
          _el$21.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$21, createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$21;
        })(), (() => {
          var _el$22 = _tmpl$6$1();
          _el$22.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$22, createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$22;
        })()] : _tmpl$7$1();
      })());
      `var(${hunkContentBGName})` != null ? _el$13.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$13.style.removeProperty("background-color");
      `var(${hunkContentColorName})` != null ? _el$14.style.setProperty("color", `var(${hunkContentColorName})`) : _el$14.style.removeProperty("color");
      insert(_el$14, () => {
        var _a, _b, _c;
        return ((_b = (_a = currentHunk()) == null ? void 0 : _a.splitInfo) == null ? void 0 : _b.plainText) || ((_c = currentHunk()) == null ? void 0 : _c.text);
      });
      `var(${hunkLineNumberBGName})` != null ? _el$15.style.setProperty("background-color", `var(${hunkLineNumberBGName})`) : _el$15.style.removeProperty("background-color");
      `var(${plainLineNumberColorName})` != null ? _el$15.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$15.style.removeProperty("color");
      `var(${borderColorName})` != null ? _el$15.style.setProperty("border-left-color", `var(${borderColorName})`) : _el$15.style.removeProperty("border-left-color");
      _el$15.style.setProperty("border-left-style", "solid");
      insert(_el$15, (() => {
        var _c$3 = createMemo(() => !!couldExpand());
        return () => _c$3() ? createMemo(() => !!currentIsFirstLine())() ? (() => {
          var _el$24 = _tmpl$2$4();
          _el$24.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$24, createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$24;
        })() : createMemo(() => !!currentIsLastLine())() ? (() => {
          var _el$25 = _tmpl$3$2();
          _el$25.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$25, createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$25;
        })() : createMemo(() => !!currentShowExpandAll())() ? (() => {
          var _el$26 = _tmpl$4$2();
          _el$26.$$click = () => props.diffFile.onSplitHunkExpand("all", props.index);
          insert(_el$26, createComponent(ExpandAll, {
            "class": "fill-current"
          }));
          return _el$26;
        })() : [(() => {
          var _el$27 = _tmpl$5$2();
          _el$27.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$27, createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$27;
        })(), (() => {
          var _el$28 = _tmpl$6$1();
          _el$28.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$28, createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$28;
        })()] : _tmpl$7$1();
      })());
      `var(${hunkContentBGName})` != null ? _el$16.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$16.style.removeProperty("background-color");
      `var(${hunkContentColorName})` != null ? _el$17.style.setProperty("color", `var(${hunkContentColorName})`) : _el$17.style.removeProperty("color");
      insert(_el$17, () => {
        var _a, _b, _c;
        return ((_b = (_a = currentHunk()) == null ? void 0 : _a.splitInfo) == null ? void 0 : _b.plainText) || ((_c = currentHunk()) == null ? void 0 : _c.text);
      });
      createRenderEffect(() => setAttribute(_el$11, "data-line", `${props.lineNumber}-hunk`));
      return _el$11;
    }
  });
};
const DiffSplitHunkLine = (props) => {
  const mode = useMode();
  return createComponent(Show, {
    get when() {
      return mode() === DiffModeEnum.SplitGitHub || mode() === DiffModeEnum.Split;
    },
    get fallback() {
      return createComponent(DiffSplitHunkLineGitLab, {
        get index() {
          return props.index;
        },
        get diffFile() {
          return props.diffFile;
        },
        get lineNumber() {
          return props.lineNumber;
        }
      });
    },
    get children() {
      return createComponent(DiffSplitHunkLineGitHub, {
        get index() {
          return props.index;
        },
        get diffFile() {
          return props.diffFile;
        },
        get lineNumber() {
          return props.lineNumber;
        }
      });
    }
  });
};
delegateEvents(["click"]);
var _tmpl$$7 = /* @__PURE__ */ template(`<tr data-state=widget class="diff-line diff-line-widget">`), _tmpl$2$3 = /* @__PURE__ */ template(`<td class="diff-line-widget-old-content p-0"colspan=2><div class=diff-line-widget-wrapper>`), _tmpl$3$1 = /* @__PURE__ */ template(`<td class="diff-line-widget-old-placeholder select-none p-0"colspan=2>`), _tmpl$4$1 = /* @__PURE__ */ template(`<td class="diff-line-widget-new-content border-l-[1px] p-0"colspan=2><div class=diff-line-widget-wrapper>`), _tmpl$5$1 = /* @__PURE__ */ template(`<td class="diff-line-widget-new-placeholder select-none border-l-[1px] p-0"colspan=2>`);
const DiffSplitWidgetLine = (props) => {
  const renderWidget = useRenderWidget();
  const [widget, setWidget] = useDiffWidgetContext() || [];
  const oldLine = createMemo(() => props.diffFile.getSplitLeftLine(props.index));
  const newLine = createMemo(() => props.diffFile.getSplitRightLine(props.index));
  const oldLineWidget = createMemo(() => {
    var _a, _b, _c, _d;
    return ((_a = oldLine()) == null ? void 0 : _a.lineNumber) && ((_b = widget == null ? void 0 : widget()) == null ? void 0 : _b.side) === SplitSide.old && ((_c = widget == null ? void 0 : widget()) == null ? void 0 : _c.lineNumber) === ((_d = oldLine()) == null ? void 0 : _d.lineNumber);
  });
  const newLineWidget = createMemo(() => {
    var _a, _b, _c, _d;
    return ((_a = newLine()) == null ? void 0 : _a.lineNumber) && ((_b = widget == null ? void 0 : widget()) == null ? void 0 : _b.side) === SplitSide.new && ((_c = widget == null ? void 0 : widget()) == null ? void 0 : _c.lineNumber) === ((_d = newLine()) == null ? void 0 : _d.lineNumber);
  });
  const currentIsShow = createMemo(() => {
    var _a, _b;
    return (!!oldLineWidget() || !!newLineWidget()) && !((_a = oldLine()) == null ? void 0 : _a.isHidden) && !((_b = newLine()) == null ? void 0 : _b.isHidden) && !!renderWidget;
  });
  const onCloseWidget = () => setWidget == null ? void 0 : setWidget({});
  return createComponent(Show, {
    get when() {
      return currentIsShow();
    },
    get children() {
      var _el$ = _tmpl$$7();
      insert(_el$, (() => {
        var _c$ = createMemo(() => !!(oldLineWidget() && renderWidget()));
        return () => _c$() ? (() => {
          var _el$2 = _tmpl$2$3(), _el$3 = _el$2.firstChild;
          insert(_el$3, () => {
            var _a, _b;
            return (_b = renderWidget == null ? void 0 : renderWidget()) == null ? void 0 : _b({
              diffFile: props.diffFile,
              side: SplitSide.old,
              lineNumber: ((_a = oldLine()) == null ? void 0 : _a.lineNumber) || 0,
              onClose: onCloseWidget
            });
          });
          return _el$2;
        })() : (() => {
          var _el$4 = _tmpl$3$1();
          `var(${emptyBGName})` != null ? _el$4.style.setProperty("background-color", `var(${emptyBGName})`) : _el$4.style.removeProperty("background-color");
          return _el$4;
        })();
      })(), null);
      insert(_el$, (() => {
        var _c$2 = createMemo(() => !!(newLineWidget() && renderWidget()));
        return () => _c$2() ? (() => {
          var _el$5 = _tmpl$4$1(), _el$6 = _el$5.firstChild;
          `var(${borderColorName})` != null ? _el$5.style.setProperty("border-left-color", `var(${borderColorName})`) : _el$5.style.removeProperty("border-left-color");
          _el$5.style.setProperty("border-left-style", "solid");
          insert(_el$6, () => {
            var _a, _b;
            return (_b = renderWidget == null ? void 0 : renderWidget()) == null ? void 0 : _b({
              diffFile: props.diffFile,
              side: SplitSide.new,
              lineNumber: ((_a = newLine()) == null ? void 0 : _a.lineNumber) || 0,
              onClose: onCloseWidget
            });
          });
          return _el$5;
        })() : (() => {
          var _el$7 = _tmpl$5$1();
          `var(${emptyBGName})` != null ? _el$7.style.setProperty("background-color", `var(${emptyBGName})`) : _el$7.style.removeProperty("background-color");
          `var(${borderColorName})` != null ? _el$7.style.setProperty("border-left-color", `var(${borderColorName})`) : _el$7.style.removeProperty("border-left-color");
          _el$7.style.setProperty("border-left-style", "solid");
          return _el$7;
        })();
      })(), null);
      createRenderEffect(() => setAttribute(_el$, "data-line", `${props.lineNumber}-widget`));
      return _el$;
    }
  });
};
var _tmpl$$6 = /* @__PURE__ */ template(`<div class="split-diff-view split-diff-view-warp w-full"><div class="diff-table-wrapper w-full"><style data-select-style></style><table class="diff-table w-full table-fixed border-collapse border-spacing-0"><colgroup><col class=diff-table-old-num-col><col class=diff-table-old-content-col><col class=diff-table-new-num-col><col class=diff-table-new-content-col></colgroup><thead class=hidden><tr><th scope=col>old line number</th><th scope=col>old line content</th><th scope=col>new line number</th><th scope=col>new line content</th></tr></thead><tbody class="diff-table-body leading-[1.4]">`);
const DiffSplitViewWrap = (props) => {
  const getAllLines = () => getSplitContentLines(props.diffFile);
  const [lines, setLines] = createSignal(getAllLines());
  const [styleRef, setStyleRef] = createSignal();
  const maxText = createMemo(() => Math.max(props.diffFile.splitLineLength, props.diffFile.fileLineLength).toString());
  const fontSize = useFontSize();
  const font = createMemo(() => ({
    fontSize: `${fontSize() || 14}px`,
    fontFamily: "Menlo, Consolas, monospace"
  }));
  createEffect(() => {
    const init = () => setLines(getAllLines);
    init();
    const cb = props.diffFile.subscribe(init);
    onCleanup(cb);
  });
  const selectState = {
    current: void 0
  };
  const onSelect = (side) => {
    const ele = styleRef();
    if (!ele) return;
    if (side) {
      const targetSide = side === SplitSide.old ? SplitSide.new : SplitSide.old;
      ele.textContent = `#diff-root${props.diffFile.getId()} [data-side="${SplitSide[targetSide]}"] {user-select: none} 
#diff-root${props.diffFile.getId()} [data-state="extend"] {user-select: none} 
#diff-root${props.diffFile.getId()} [data-state="hunk"] {user-select: none} 
#diff-root${props.diffFile.getId()} [data-state="widget"] {user-select: none}`;
    } else {
      ele.textContent = "";
    }
  };
  const onMouseDown = (e) => {
    let ele = e.target;
    if (ele && ele instanceof HTMLElement && ele.nodeName === "BUTTON") {
      removeAllSelection();
      return;
    }
    while (ele && ele instanceof HTMLElement) {
      const state = ele.getAttribute("data-state");
      const side = ele.getAttribute("data-side");
      if (side) {
        if (selectState.current !== SplitSide[side]) {
          selectState.current = SplitSide[side];
          onSelect(SplitSide[side]);
          removeAllSelection();
        }
      }
      if (state) {
        if (state === "extend" || state === "hunk" || state === "widget") {
          if (selectState.current !== void 0) {
            selectState.current = void 0;
            onSelect(void 0);
            removeAllSelection();
          }
          return;
        } else {
          return;
        }
      }
      ele = ele.parentElement;
    }
  };
  const width = useTextWidth({
    text: maxText,
    font
  });
  const memoWidth = createMemo(() => Math.max(40, width() + 25));
  return (() => {
    var _el$ = _tmpl$$6(), _el$2 = _el$.firstChild, _el$3 = _el$2.firstChild, _el$4 = _el$3.nextSibling, _el$5 = _el$4.firstChild, _el$6 = _el$5.firstChild, _el$7 = _el$6.nextSibling, _el$8 = _el$7.nextSibling, _el$9 = _el$5.nextSibling, _el$10 = _el$9.nextSibling;
    _el$2.style.setProperty("font-family", "Menlo, Consolas, monospace");
    `var(${diffFontSizeName})` != null ? _el$2.style.setProperty("font-size", `var(${diffFontSizeName})`) : _el$2.style.removeProperty("font-size");
    use(setStyleRef, _el$3);
    _el$10.$$mousedown = onMouseDown;
    insert(_el$10, createComponent(For, {
      get each() {
        return lines();
      },
      children: (item) => [createComponent(DiffSplitHunkLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), createComponent(DiffSplitContentLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), createComponent(DiffSplitWidgetLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), createComponent(DiffSplitExtendLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      })]
    }), null);
    insert(_el$10, createComponent(DiffSplitHunkLine, {
      get index() {
        return props.diffFile.splitLineLength;
      },
      get lineNumber() {
        return props.diffFile.splitLineLength;
      },
      get diffFile() {
        return props.diffFile;
      }
    }), null);
    createRenderEffect((_p$) => {
      var _v$ = {
        [diffAsideWidthName]: `${Math.round(memoWidth())}px`
      }, _v$2 = Math.round(memoWidth()), _v$3 = Math.round(memoWidth());
      _p$.e = style(_el$2, _v$, _p$.e);
      _v$2 !== _p$.t && setAttribute(_el$6, "width", _p$.t = _v$2);
      _v$3 !== _p$.a && setAttribute(_el$8, "width", _p$.a = _v$3);
      return _p$;
    }, {
      e: void 0,
      t: void 0,
      a: void 0
    });
    return _el$;
  })();
};
delegateEvents(["mousedown"]);
const DiffSplitView = (props) => {
  const enableWrap = useEnableWrap();
  return createComponent(Show, {
    get when() {
      return enableWrap();
    },
    get fallback() {
      return createComponent(DiffSplitViewNormal, {
        get diffFile() {
          return props.diffFile;
        }
      });
    },
    get children() {
      return createComponent(DiffSplitViewWrap, {
        get diffFile() {
          return props.diffFile;
        }
      });
    }
  });
};
var _tmpl$$5 = /* @__PURE__ */ template(`<tr data-state=diff class="diff-line group"><td class="diff-line-num sticky left-0 z-[1] w-[1%] min-w-[100px] select-none whitespace-nowrap pl-[10px] pr-[10px] text-right align-top"><div class=flex><span class="inline-block w-[50%]"></span><span class="w-[10px] shrink-0"></span><span class="inline-block w-[50%]"></span></div></td><td class="diff-line-content pr-[10px] align-top">`), _tmpl$2$2 = /* @__PURE__ */ template(`<tr class="diff-line group"><td class="diff-line-num sticky left-0 z-[1] w-[1%] min-w-[100px] select-none whitespace-nowrap pl-[10px] pr-[10px] text-right align-top"><div class="flex opacity-[0.5]"><span class="inline-block w-[50%]"></span><span class="w-[10px] shrink-0"></span><span class="inline-block w-[50%]"></span></div></td><td class="diff-line-content pr-[10px] align-top">`);
const DiffUnifiedOldLine = (props) => {
  return (() => {
    var _el$ = _tmpl$$5(), _el$2 = _el$.firstChild, _el$3 = _el$2.firstChild, _el$4 = _el$3.firstChild, _el$5 = _el$2.nextSibling;
    `var(${plainLineNumberColorName})` != null ? _el$2.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$2.style.removeProperty("color");
    `var(${delLineNumberBGName})` != null ? _el$2.style.setProperty("background-color", `var(${delLineNumberBGName})`) : _el$2.style.removeProperty("background-color");
    `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$2.style.setProperty("width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$2.style.removeProperty("width");
    `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$2.style.setProperty("max-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$2.style.removeProperty("max-width");
    `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$2.style.setProperty("min-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$2.style.removeProperty("min-width");
    insert(_el$2, (() => {
      var _c$ = createMemo(() => !!props.enableAddWidget);
      return () => _c$() && createComponent(DiffUnifiedAddWidget, {
        get index() {
          return props.index - 1;
        },
        get lineNumber() {
          return props.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        },
        get side() {
          return SplitSide.old;
        },
        get onWidgetClick() {
          return props.onAddWidgetClick;
        },
        get onOpenAddWidget() {
          return props.onOpenAddWidget;
        }
      });
    })(), _el$3);
    insert(_el$4, () => props.lineNumber);
    `var(${delContentBGName})` != null ? _el$5.style.setProperty("background-color", `var(${delContentBGName})`) : _el$5.style.removeProperty("background-color");
    insert(_el$5, createComponent(DiffContent, {
      get enableWrap() {
        return props.enableWrap;
      },
      get diffFile() {
        return props.diffFile;
      },
      get enableHighlight() {
        return props.enableHighlight;
      },
      get rawLine() {
        return props.rawLine;
      },
      get diffLine() {
        return props.diffLine;
      },
      get plainLine() {
        return props.plainLine;
      },
      get syntaxLine() {
        return props.syntaxLine;
      }
    }));
    createRenderEffect((_p$) => {
      var _v$ = props.index, _v$2 = props.lineNumber;
      _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
      _v$2 !== _p$.t && setAttribute(_el$4, "data-line-old-num", _p$.t = _v$2);
      return _p$;
    }, {
      e: void 0,
      t: void 0
    });
    return _el$;
  })();
};
const DiffUnifiedNewLine = (props) => {
  return (() => {
    var _el$6 = _tmpl$$5(), _el$7 = _el$6.firstChild, _el$8 = _el$7.firstChild, _el$9 = _el$8.firstChild, _el$10 = _el$9.nextSibling, _el$11 = _el$10.nextSibling, _el$12 = _el$7.nextSibling;
    `var(${plainLineNumberColorName})` != null ? _el$7.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$7.style.removeProperty("color");
    `var(${addLineNumberBGName})` != null ? _el$7.style.setProperty("background-color", `var(${addLineNumberBGName})`) : _el$7.style.removeProperty("background-color");
    `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$7.style.setProperty("width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$7.style.removeProperty("width");
    `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$7.style.setProperty("max-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$7.style.removeProperty("max-width");
    `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$7.style.setProperty("min-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$7.style.removeProperty("min-width");
    insert(_el$7, (() => {
      var _c$2 = createMemo(() => !!props.enableAddWidget);
      return () => _c$2() && createComponent(DiffUnifiedAddWidget, {
        get index() {
          return props.index - 1;
        },
        get lineNumber() {
          return props.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        },
        get side() {
          return SplitSide.new;
        },
        get onWidgetClick() {
          return props.onAddWidgetClick;
        },
        get onOpenAddWidget() {
          return props.onOpenAddWidget;
        }
      });
    })(), _el$8);
    insert(_el$11, () => props.lineNumber);
    `var(${addContentBGName})` != null ? _el$12.style.setProperty("background-color", `var(${addContentBGName})`) : _el$12.style.removeProperty("background-color");
    insert(_el$12, createComponent(DiffContent, {
      get enableWrap() {
        return props.enableWrap;
      },
      get diffFile() {
        return props.diffFile;
      },
      get enableHighlight() {
        return props.enableHighlight;
      },
      get rawLine() {
        return props.rawLine;
      },
      get diffLine() {
        return props.diffLine;
      },
      get plainLine() {
        return props.plainLine;
      },
      get syntaxLine() {
        return props.syntaxLine;
      }
    }));
    createRenderEffect((_p$) => {
      var _v$3 = props.index, _v$4 = props.lineNumber;
      _v$3 !== _p$.e && setAttribute(_el$6, "data-line", _p$.e = _v$3);
      _v$4 !== _p$.t && setAttribute(_el$11, "data-line-new-num", _p$.t = _v$4);
      return _p$;
    }, {
      e: void 0,
      t: void 0
    });
    return _el$6;
  })();
};
const DiffUnifiedContentLine = (props) => {
  const unifiedItem = createMemo(() => props.diffFile.getUnifiedLine(props.index));
  const enableWrap = useEnableWrap();
  const [_, setWidget] = useDiffWidgetContext() || [];
  const onAddWidgetClick = useOnAddWidgetClick();
  const enableHighlight = useEnableHighlight();
  const enableAddWidget = useEnableAddWidget();
  const currentItemHasHidden = createMemo(() => {
    var _a;
    return (_a = unifiedItem()) == null ? void 0 : _a.isHidden;
  });
  const currentItemHasChange = createMemo(() => {
    var _a;
    return checkDiffLineIncludeChange((_a = unifiedItem()) == null ? void 0 : _a.diff);
  });
  const getCurrentSyntaxLine = () => {
    var _a, _b, _c, _d;
    return ((_a = unifiedItem()) == null ? void 0 : _a.newLineNumber) ? props.diffFile.getNewSyntaxLine(((_b = unifiedItem()) == null ? void 0 : _b.newLineNumber) || 0) : ((_c = unifiedItem()) == null ? void 0 : _c.oldLineNumber) ? props.diffFile.getOldSyntaxLine(((_d = unifiedItem()) == null ? void 0 : _d.oldLineNumber) || 0) : void 0;
  };
  const [currentSyntaxLine, setCurrentSyntaxLine] = createSignal(getCurrentSyntaxLine());
  const getCurrentPlainLine = () => {
    var _a, _b, _c, _d;
    return ((_a = unifiedItem()) == null ? void 0 : _a.newLineNumber) ? props.diffFile.getNewPlainLine(((_b = unifiedItem()) == null ? void 0 : _b.newLineNumber) || 0) : ((_c = unifiedItem()) == null ? void 0 : _c.oldLineNumber) ? props.diffFile.getOldPlainLine(((_d = unifiedItem()) == null ? void 0 : _d.oldLineNumber) || 0) : void 0;
  };
  const [currentPlainLine, setCurrentPlainLine] = createSignal(getCurrentPlainLine());
  createEffect(() => {
    const init = () => {
      setCurrentSyntaxLine(getCurrentSyntaxLine);
      setCurrentPlainLine(getCurrentPlainLine);
    };
    init();
    const unsubscribe = props.diffFile.subscribe(init);
    onCleanup(unsubscribe);
  });
  const onClickAddWidget = (lineNumber, side) => setWidget == null ? void 0 : setWidget({
    side,
    lineNumber
  });
  return createComponent(Show, {
    get when() {
      return !currentItemHasHidden();
    },
    get children() {
      return createComponent(Show, {
        get when() {
          return currentItemHasChange();
        },
        get fallback() {
          return (() => {
            var _el$13 = _tmpl$2$2(), _el$14 = _el$13.firstChild, _el$15 = _el$14.firstChild, _el$16 = _el$15.firstChild, _el$17 = _el$16.nextSibling, _el$18 = _el$17.nextSibling, _el$19 = _el$14.nextSibling;
            `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$14.style.setProperty("width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$14.style.removeProperty("width");
            `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$14.style.setProperty("max-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$14.style.removeProperty("max-width");
            `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$14.style.setProperty("min-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$14.style.removeProperty("min-width");
            insert(_el$14, (() => {
              var _c$3 = createMemo(() => {
                var _a;
                return !!(enableAddWidget() && ((_a = unifiedItem()) == null ? void 0 : _a.diff));
              });
              return () => _c$3() && createComponent(DiffUnifiedAddWidget, {
                get index() {
                  return props.index;
                },
                get diffFile() {
                  return props.diffFile;
                },
                get lineNumber() {
                  var _a;
                  return ((_a = unifiedItem()) == null ? void 0 : _a.newLineNumber) || 0;
                },
                get side() {
                  return SplitSide.new;
                },
                onOpenAddWidget: onClickAddWidget,
                onWidgetClick: onAddWidgetClick
              });
            })(), _el$15);
            insert(_el$16, () => {
              var _a;
              return ((_a = unifiedItem()) == null ? void 0 : _a.oldLineNumber) || 0;
            });
            insert(_el$18, () => {
              var _a;
              return ((_a = unifiedItem()) == null ? void 0 : _a.newLineNumber) || 0;
            });
            insert(_el$19, createComponent(DiffContent, {
              get enableWrap() {
                return !!enableWrap();
              },
              get diffFile() {
                return props.diffFile;
              },
              get enableHighlight() {
                return !!enableHighlight();
              },
              get rawLine() {
                var _a;
                return ((_a = unifiedItem()) == null ? void 0 : _a.value) || "";
              },
              get diffLine() {
                var _a;
                return (_a = unifiedItem()) == null ? void 0 : _a.diff;
              },
              get plainLine() {
                return currentPlainLine();
              },
              get syntaxLine() {
                return currentSyntaxLine();
              }
            }));
            createRenderEffect((_p$) => {
              var _a, _b, _c, _d, _e, _f;
              var _v$5 = props.lineNumber, _v$6 = ((_a = unifiedItem()) == null ? void 0 : _a.diff) ? "diff" : "plain", _v$7 = `var(${((_b = unifiedItem()) == null ? void 0 : _b.diff) ? plainLineNumberColorName : expandLineNumberColorName})`, _v$8 = ((_c = unifiedItem()) == null ? void 0 : _c.diff) ? `var(${plainLineNumberBGName})` : `var(${expandContentBGName})`, _v$9 = ((_d = unifiedItem()) == null ? void 0 : _d.oldLineNumber) || 0, _v$10 = ((_e = unifiedItem()) == null ? void 0 : _e.newLineNumber) || 0, _v$11 = ((_f = unifiedItem()) == null ? void 0 : _f.diff) ? `var(${plainContentBGName})` : `var(${expandContentBGName})`;
              _v$5 !== _p$.e && setAttribute(_el$13, "data-line", _p$.e = _v$5);
              _v$6 !== _p$.t && setAttribute(_el$13, "data-state", _p$.t = _v$6);
              _v$7 !== _p$.a && ((_p$.a = _v$7) != null ? _el$14.style.setProperty("color", _v$7) : _el$14.style.removeProperty("color"));
              _v$8 !== _p$.o && ((_p$.o = _v$8) != null ? _el$14.style.setProperty("background-color", _v$8) : _el$14.style.removeProperty("background-color"));
              _v$9 !== _p$.i && setAttribute(_el$16, "data-line-old-num", _p$.i = _v$9);
              _v$10 !== _p$.n && setAttribute(_el$18, "data-line-new-num", _p$.n = _v$10);
              _v$11 !== _p$.s && ((_p$.s = _v$11) != null ? _el$19.style.setProperty("background-color", _v$11) : _el$19.style.removeProperty("background-color"));
              return _p$;
            }, {
              e: void 0,
              t: void 0,
              a: void 0,
              o: void 0,
              i: void 0,
              n: void 0,
              s: void 0
            });
            return _el$13;
          })();
        },
        get children() {
          return createComponent(Show, {
            get when() {
              var _a;
              return (_a = unifiedItem()) == null ? void 0 : _a.oldLineNumber;
            },
            get fallback() {
              return createComponent(DiffUnifiedNewLine, {
                get index() {
                  return props.lineNumber;
                },
                get enableWrap() {
                  return !!enableWrap();
                },
                get diffFile() {
                  return props.diffFile;
                },
                get rawLine() {
                  var _a;
                  return ((_a = unifiedItem()) == null ? void 0 : _a.value) || "";
                },
                get diffLine() {
                  var _a;
                  return (_a = unifiedItem()) == null ? void 0 : _a.diff;
                },
                get plainLine() {
                  return currentPlainLine();
                },
                get syntaxLine() {
                  return currentSyntaxLine();
                },
                get enableHighlight() {
                  return !!enableHighlight();
                },
                get enableAddWidget() {
                  return !!enableAddWidget();
                },
                get lineNumber() {
                  return unifiedItem().newLineNumber || 0;
                },
                onAddWidgetClick,
                onOpenAddWidget: onClickAddWidget
              });
            },
            get children() {
              return createComponent(DiffUnifiedOldLine, {
                get index() {
                  return props.lineNumber;
                },
                get enableWrap() {
                  return !!enableWrap();
                },
                get diffFile() {
                  return props.diffFile;
                },
                get rawLine() {
                  var _a;
                  return ((_a = unifiedItem()) == null ? void 0 : _a.value) || "";
                },
                get diffLine() {
                  var _a;
                  return (_a = unifiedItem()) == null ? void 0 : _a.diff;
                },
                get plainLine() {
                  return currentPlainLine();
                },
                get syntaxLine() {
                  return currentSyntaxLine();
                },
                get enableHighlight() {
                  return !!enableHighlight();
                },
                get enableAddWidget() {
                  return !!enableAddWidget();
                },
                get lineNumber() {
                  return unifiedItem().oldLineNumber || 0;
                },
                onAddWidgetClick,
                onOpenAddWidget: onClickAddWidget
              });
            }
          });
        }
      });
    }
  });
};
var _tmpl$$4 = /* @__PURE__ */ template(`<tr data-state=extend class="diff-line diff-line-extend"><td class="diff-line-extend-content p-0 align-top"colspan=2><div class="diff-line-extend-wrapper sticky left-0 z-[1]">`);
const DiffUnifiedExtendLine = (props) => {
  const extendData = useExtendData();
  const renderExtend = useRenderExtend();
  const unifiedItem = createMemo(() => props.diffFile.getUnifiedLine(props.index));
  const oldExtend = createMemo(() => {
    var _a, _b, _c;
    return (_c = (_a = extendData()) == null ? void 0 : _a.oldFile) == null ? void 0 : _c[((_b = unifiedItem()) == null ? void 0 : _b.oldLineNumber) || 0];
  });
  const newExtend = createMemo(() => {
    var _a, _b, _c;
    return (_c = (_a = extendData()) == null ? void 0 : _a.newFile) == null ? void 0 : _c[((_b = unifiedItem()) == null ? void 0 : _b.newLineNumber) || 0];
  });
  const currentIsHidden = createMemo(() => {
    var _a;
    return (_a = unifiedItem()) == null ? void 0 : _a.isHidden;
  });
  const currentIsShow = createMemo(() => Boolean((oldExtend() || newExtend()) && !currentIsHidden() && renderExtend()));
  const lineSelector = createMemo(() => `.unified-diff-table-wrapper`);
  const width = useDomWidth({
    selector: lineSelector,
    enable: currentIsShow
  });
  return createComponent(Show, {
    get when() {
      return currentIsShow();
    },
    get children() {
      var _el$ = _tmpl$$4(), _el$2 = _el$.firstChild, _el$3 = _el$2.firstChild;
      insert(_el$3, (() => {
        var _c$ = createMemo(() => !!(width() > 0 && oldExtend()));
        return () => {
          var _a, _b, _c;
          return _c$() && ((_c = renderExtend()) == null ? void 0 : _c({
            diffFile: props.diffFile,
            side: SplitSide.old,
            lineNumber: ((_a = unifiedItem()) == null ? void 0 : _a.oldLineNumber) || 0,
            data: (_b = oldExtend()) == null ? void 0 : _b.data,
            onUpdate: props.diffFile.notifyAll
          }));
        };
      })(), null);
      insert(_el$3, (() => {
        var _c$2 = createMemo(() => !!(width() > 0 && newExtend()));
        return () => {
          var _a, _b;
          return _c$2() && ((_b = renderExtend()) == null ? void 0 : _b({
            diffFile: props.diffFile,
            side: SplitSide.new,
            lineNumber: unifiedItem().newLineNumber || 0,
            data: (_a = newExtend()) == null ? void 0 : _a.data,
            onUpdate: props.diffFile.notifyAll
          }));
        };
      })(), null);
      createRenderEffect((_p$) => {
        var _v$ = `${props.lineNumber}-extend`, _v$2 = width() + "px";
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && ((_p$.t = _v$2) != null ? _el$3.style.setProperty("width", _v$2) : _el$3.style.removeProperty("width"));
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$;
    }
  });
};
var _tmpl$$3 = /* @__PURE__ */ template(`<tr data-state=hunk class="diff-line diff-line-hunk"><td class="diff-line-hunk-action sticky left-0 w-[1%] min-w-[100px] select-none"></td><td class="diff-line-hunk-content pr-[10px] align-middle"><div class=pl-[1.5em]>`), _tmpl$2$1 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand Up"data-title="Expand Up">`), _tmpl$3 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand Down"data-title="Expand Down">`), _tmpl$4 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand All"data-title="Expand All">`), _tmpl$5 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[2px]"title="Expand Down"data-title="Expand Down">`), _tmpl$6 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[2px]"title="Expand Up"data-title="Expand Up">`), _tmpl$7 = /* @__PURE__ */ template(`<div class=min-h-[28px]>&ensp;`);
const DiffUnifiedHunkLine = (props) => {
  const currentHunk = createMemo(() => props.diffFile.getUnifiedHunkLine(props.index));
  const enableExpand = createMemo(() => props.diffFile.getExpandEnabled());
  const couldExpand = createMemo(() => {
    var _a;
    return enableExpand() && !!((_a = currentHunk()) == null ? void 0 : _a.unifiedInfo);
  });
  const enableWrap = useEnableWrap();
  const checkCurrentIsShow = () => {
    const hunk = currentHunk();
    return hunk && hunk.unifiedInfo && hunk.unifiedInfo.startHiddenIndex < hunk.unifiedInfo.endHiddenIndex;
  };
  const [currentIsShow, setCurrentIsShow] = createSignal(checkCurrentIsShow());
  const checkCurrentIsEnableAll = () => {
    const hunk = currentHunk();
    return hunk && hunk.unifiedInfo && hunk.unifiedInfo.endHiddenIndex - hunk.unifiedInfo.startHiddenIndex < composeLen;
  };
  const [currentIsEnableAll, setCurrentIsEnableAll] = createSignal(checkCurrentIsEnableAll());
  const currentIsFirstLine = createMemo(() => {
    var _a;
    return (_a = currentHunk()) == null ? void 0 : _a.isFirst;
  });
  const currentIsLastLine = createMemo(() => {
    var _a;
    return (_a = currentHunk()) == null ? void 0 : _a.isLast;
  });
  const currentIsPureHunk = createMemo(() => {
    var _a;
    return currentHunk() && props.diffFile._getIsPureDiffRender() && !((_a = currentHunk()) == null ? void 0 : _a.unifiedInfo);
  });
  createEffect(() => {
    const init = () => {
      setCurrentIsShow(checkCurrentIsShow);
      setCurrentIsEnableAll(checkCurrentIsEnableAll);
    };
    init();
    const unsubscribe = props.diffFile.subscribe(init);
    onCleanup(unsubscribe);
  });
  return createComponent(Show, {
    get when() {
      return currentIsShow() || currentIsPureHunk();
    },
    get children() {
      var _el$ = _tmpl$$3(), _el$2 = _el$.firstChild, _el$3 = _el$2.nextSibling, _el$4 = _el$3.firstChild;
      `var(${hunkLineNumberBGName})` != null ? _el$2.style.setProperty("background-color", `var(${hunkLineNumberBGName})`) : _el$2.style.removeProperty("background-color");
      `var(${plainLineNumberColorName})` != null ? _el$2.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$2.style.removeProperty("color");
      `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$2.style.setProperty("width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$2.style.removeProperty("width");
      `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$2.style.setProperty("max-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$2.style.removeProperty("max-width");
      `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$2.style.setProperty("min-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$2.style.removeProperty("min-width");
      insert(_el$2, (() => {
        var _c$ = createMemo(() => !!couldExpand());
        return () => _c$() ? createMemo(() => !!currentIsFirstLine())() ? (() => {
          var _el$5 = _tmpl$2$1();
          _el$5.$$click = () => props.diffFile.onUnifiedHunkExpand("up", props.index);
          insert(_el$5, createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$5;
        })() : createMemo(() => !!currentIsLastLine())() ? (() => {
          var _el$6 = _tmpl$3();
          _el$6.$$click = () => props.diffFile.onUnifiedHunkExpand("down", props.index);
          insert(_el$6, createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$6;
        })() : createMemo(() => !!currentIsEnableAll())() ? (() => {
          var _el$7 = _tmpl$4();
          _el$7.$$click = () => props.diffFile.onUnifiedHunkExpand("all", props.index);
          insert(_el$7, createComponent(ExpandAll, {
            "class": "fill-current"
          }));
          return _el$7;
        })() : [(() => {
          var _el$8 = _tmpl$5();
          _el$8.$$click = () => props.diffFile.onUnifiedHunkExpand("down", props.index);
          insert(_el$8, createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$8;
        })(), (() => {
          var _el$9 = _tmpl$6();
          _el$9.$$click = () => props.diffFile.onUnifiedHunkExpand("up", props.index);
          insert(_el$9, createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$9;
        })()] : _tmpl$7();
      })());
      `var(${hunkContentBGName})` != null ? _el$3.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$3.style.removeProperty("background-color");
      `var(${hunkContentColorName})` != null ? _el$4.style.setProperty("color", `var(${hunkContentColorName})`) : _el$4.style.removeProperty("color");
      insert(_el$4, () => {
        var _a, _b, _c;
        return ((_b = (_a = currentHunk()) == null ? void 0 : _a.unifiedInfo) == null ? void 0 : _b.plainText) || ((_c = currentHunk()) == null ? void 0 : _c.text);
      });
      createRenderEffect((_p$) => {
        var _v$ = `${props.lineNumber}-hunk`, _v$2 = enableWrap() ? "pre-wrap" : "pre", _v$3 = enableWrap() ? "break-all" : "initial";
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && ((_p$.t = _v$2) != null ? _el$4.style.setProperty("white-space", _v$2) : _el$4.style.removeProperty("white-space"));
        _v$3 !== _p$.a && ((_p$.a = _v$3) != null ? _el$4.style.setProperty("word-break", _v$3) : _el$4.style.removeProperty("word-break"));
        return _p$;
      }, {
        e: void 0,
        t: void 0,
        a: void 0
      });
      return _el$;
    }
  });
};
delegateEvents(["click"]);
var _tmpl$$2 = /* @__PURE__ */ template(`<tr data-state=widget class="diff-line diff-line-widget"><td class="diff-line-widget-content p-0"colspan=2><div class="diff-line-widget-wrapper sticky left-0 z-[1]">`);
const DiffUnifiedWidgetLine = (props) => {
  const renderWidget = useRenderWidget();
  const [widget, setWidget] = useDiffWidgetContext() || [];
  const unifiedItem = createMemo(() => props.diffFile.getUnifiedLine(props.index));
  const oldWidget = createMemo(() => {
    var _a, _b, _c, _d;
    return ((_a = unifiedItem()) == null ? void 0 : _a.oldLineNumber) && ((_b = widget == null ? void 0 : widget()) == null ? void 0 : _b.side) === SplitSide.old && ((_c = widget == null ? void 0 : widget()) == null ? void 0 : _c.lineNumber) === ((_d = unifiedItem()) == null ? void 0 : _d.oldLineNumber);
  });
  const newWidget = createMemo(() => {
    var _a, _b, _c, _d;
    return ((_a = unifiedItem()) == null ? void 0 : _a.newLineNumber) && ((_b = widget == null ? void 0 : widget()) == null ? void 0 : _b.side) === SplitSide.new && ((_c = widget == null ? void 0 : widget()) == null ? void 0 : _c.lineNumber) === ((_d = unifiedItem()) == null ? void 0 : _d.newLineNumber);
  });
  const currentIsHidden = createMemo(() => {
    var _a;
    return (_a = unifiedItem()) == null ? void 0 : _a.isHidden;
  });
  const currenIsShow = createMemo(() => !!(oldWidget() || newWidget()) && !currentIsHidden() && !!renderWidget());
  const onCloseWidget = () => setWidget == null ? void 0 : setWidget({});
  const lineSelector = createMemo(() => `.unified-diff-table-wrapper`);
  const width = useDomWidth({
    selector: lineSelector,
    enable: currenIsShow
  });
  return createComponent(Show, {
    get when() {
      return currenIsShow();
    },
    get children() {
      var _el$ = _tmpl$$2(), _el$2 = _el$.firstChild, _el$3 = _el$2.firstChild;
      insert(_el$3, (() => {
        var _c$ = createMemo(() => !!(width() > 0 && oldWidget()));
        return () => {
          var _a, _b;
          return _c$() && ((_b = renderWidget()) == null ? void 0 : _b({
            diffFile: props.diffFile,
            side: SplitSide.old,
            lineNumber: ((_a = unifiedItem()) == null ? void 0 : _a.oldLineNumber) || 0,
            onClose: onCloseWidget
          }));
        };
      })(), null);
      insert(_el$3, (() => {
        var _c$2 = createMemo(() => !!(width() > 0 && newWidget()));
        return () => {
          var _a;
          return _c$2() && ((_a = renderWidget()) == null ? void 0 : _a({
            diffFile: props.diffFile,
            side: SplitSide.new,
            lineNumber: unifiedItem().newLineNumber || 0,
            onClose: onCloseWidget
          }));
        };
      })(), null);
      createRenderEffect((_p$) => {
        var _v$ = `${props.lineNumber}-widget`, _v$2 = width() + "px";
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && ((_p$.t = _v$2) != null ? _el$3.style.setProperty("width", _v$2) : _el$3.style.removeProperty("width"));
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$;
    }
  });
};
var _tmpl$$1 = /* @__PURE__ */ template(`<div><style data-select-style></style><div class="unified-diff-table-wrapper diff-table-scroll-container w-full overflow-x-auto overflow-y-hidden"><table><colgroup><col class=unified-diff-table-num-col><col class=unified-diff-table-content-col></colgroup><thead class=hidden><tr><th scope=col>line number</th><th scope=col>line content</th></tr></thead><tbody class="diff-table-body leading-[1.4]">`);
const DiffUnifiedView = (props) => {
  const getAllLines = () => getUnifiedContentLine(props.diffFile);
  const [lines, setLines] = createSignal(getAllLines());
  const maxText = createMemo(() => Math.max(props.diffFile.unifiedLineLength, props.diffFile.fileLineLength).toString());
  createEffect(() => {
    const init = () => setLines(getAllLines);
    init();
    const unsubscribe = props.diffFile.subscribe(init);
    onCleanup(unsubscribe);
  });
  const fontSize = useFontSize();
  const enableWrap = useEnableWrap();
  const [styleRef, setStyleRef] = createSignal();
  const selectState = {
    current: void 0
  };
  const onSelect = (s) => {
    const ele = styleRef();
    if (!ele) return;
    if (s) {
      ele.textContent = `#${getId()} [data-state="extend"] {user-select: none} 
#${getId()} [data-state="hunk"] {user-select: none} 
#${getId()} [data-state="widget"] {user-select: none}`;
    } else {
      ele.textContent = "";
    }
  };
  const onMouseDown = (e) => {
    let ele = e.target;
    if (ele && (ele == null ? void 0 : ele.nodeName) === "BUTTON") {
      removeAllSelection();
      return;
    }
    while (ele && ele instanceof HTMLElement) {
      const state = ele.getAttribute("data-state");
      if (state) {
        if (state === "extend" || state === "hunk" || state === "widget") {
          if (selectState.current !== false) {
            selectState.current = false;
            onSelect(false);
            removeAllSelection();
          }
        } else {
          if (selectState.current !== true) {
            selectState.current = true;
            onSelect(true);
            removeAllSelection();
          }
        }
        return;
      }
      ele = ele.parentElement;
    }
  };
  const font = createMemo(() => ({
    fontSize: fontSize() + "px",
    fontFamily: "Menlo, Consolas, monospace"
  }));
  const width = useTextWidth({
    text: maxText,
    font
  });
  const computedWidth = createMemo(() => Math.max(40, width() + 10));
  const getId = () => `diff-root${props.diffFile.getId()}`;
  return (() => {
    var _el$ = _tmpl$$1(), _el$2 = _el$.firstChild, _el$3 = _el$2.nextSibling, _el$4 = _el$3.firstChild, _el$5 = _el$4.firstChild, _el$6 = _el$5.nextSibling, _el$7 = _el$6.nextSibling;
    use(setStyleRef, _el$2);
    _el$3.style.setProperty("font-family", "Menlo, Consolas, monospace");
    `var(${diffFontSizeName})` != null ? _el$3.style.setProperty("font-size", `var(${diffFontSizeName})`) : _el$3.style.removeProperty("font-size");
    _el$7.$$mousedown = onMouseDown;
    insert(_el$7, createComponent(For, {
      get each() {
        return lines();
      },
      children: (item) => [createComponent(DiffUnifiedHunkLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), createComponent(DiffUnifiedContentLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), createComponent(DiffUnifiedWidgetLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), createComponent(DiffUnifiedExtendLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      })]
    }), null);
    insert(_el$7, createComponent(DiffUnifiedHunkLine, {
      get index() {
        return props.diffFile.unifiedLineLength;
      },
      get lineNumber() {
        return props.diffFile.unifiedLineLength;
      },
      get diffFile() {
        return props.diffFile;
      }
    }), null);
    createRenderEffect((_p$) => {
      var _v$ = `unified-diff-view ${enableWrap() ? "unified-diff-view-wrap" : "unified-diff-view-normal"} w-full`, _v$2 = {
        [diffAsideWidthName]: `${Math.round(computedWidth())}px`
      }, _v$3 = `unified-diff-table w-full border-collapse border-spacing-0 ${enableWrap() ? "table-fixed" : ""}`;
      _v$ !== _p$.e && className(_el$, _p$.e = _v$);
      _p$.t = style(_el$3, _v$2, _p$.t);
      _v$3 !== _p$.a && className(_el$4, _p$.a = _v$3);
      return _p$;
    }, {
      e: void 0,
      t: void 0,
      a: void 0
    });
    return _el$;
  })();
};
delegateEvents(["mousedown"]);
const createDiffConfigStore = (props, diffFileId) => {
  return createStore(() => {
    var _a, _b;
    const id = ref(diffFileId);
    const setId = (_id) => id.value = _id;
    const mode = ref(props.diffViewMode);
    const setMode = (_mode) => mode.value = _mode;
    const isMounted = ref(false);
    const setIsIsMounted = (_isMounted) => isMounted.value = _isMounted;
    const enableWrap = ref(props.diffViewWrap);
    const setEnableWrap = (_enableWrap) => enableWrap.value = _enableWrap;
    const enableAddWidget = ref(props.diffViewAddWidget);
    const setEnableAddWidget = (_enableAddWidget) => enableAddWidget.value = _enableAddWidget;
    const enableHighlight = ref(props.diffViewHighlight);
    const setEnableHighlight = (_enableHighlight) => enableHighlight.value = _enableHighlight;
    const fontSize = ref(props.diffViewFontSize);
    const setFontSize = (_fontSize) => fontSize.value = _fontSize;
    const extendData = ref({
      oldFile: { ...(_a = props.extendData) == null ? void 0 : _a.oldFile },
      newFile: { ...(_b = props.extendData) == null ? void 0 : _b.newFile }
    });
    const setExtendData = (_extendData) => {
      extendData.value = { oldFile: { ..._extendData == null ? void 0 : _extendData.oldFile }, newFile: { ..._extendData == null ? void 0 : _extendData.newFile } };
    };
    const renderWidgetLine = ref(props.renderWidgetLine);
    const setRenderWidgetLine = (_renderWidgetLine) => renderWidgetLine.value = _renderWidgetLine;
    const renderExtendLine = ref(props.renderExtendLine);
    const setRenderExtendLine = (_renderExtendLine) => renderExtendLine.value = _renderExtendLine;
    const onAddWidgetClick = { current: props.onAddWidgetClick };
    const setOnAddWidgetClick = (_onAddWidgetClick) => onAddWidgetClick.current = _onAddWidgetClick.current;
    return {
      id,
      setId,
      mode,
      setMode,
      isMounted,
      setIsIsMounted,
      enableWrap,
      setEnableWrap,
      enableAddWidget,
      setEnableAddWidget,
      enableHighlight,
      setEnableHighlight,
      fontSize,
      setFontSize,
      extendData,
      setExtendData,
      renderWidgetLine,
      setRenderWidgetLine,
      renderExtendLine,
      setRenderExtendLine,
      onAddWidgetClick,
      setOnAddWidgetClick
    };
  });
};
var _tmpl$ = /* @__PURE__ */ template(`<div class=diff-style-root><div>`), _tmpl$2 = /* @__PURE__ */ template(`<div class=diff-tailwindcss-wrapper data-component=git-diff-view>`);
_cacheMap.name = "@git-diff-view/solid";
const InternalDiffView = (props) => {
  var _a, _b, _c;
  const getInstance = () => {
    var _a2, _b2, _c2, _d, _e, _f;
    let diffFile2 = null;
    if (props.diffFile) {
      diffFile2 = DiffFile.createInstance({});
      diffFile2._mergeFullBundle(props.diffFile._getFullBundle());
    } else if (props.data) {
      diffFile2 = new DiffFile(((_a2 = props.data.oldFile) == null ? void 0 : _a2.fileName) || "", ((_b2 = props.data.oldFile) == null ? void 0 : _b2.content) || "", ((_c2 = props.data.newFile) == null ? void 0 : _c2.fileName) || "", ((_d = props.data.newFile) == null ? void 0 : _d.content) || "", props.data.hunks || [], ((_e = props.data.oldFile) == null ? void 0 : _e.fileLang) || "", ((_f = props.data.newFile) == null ? void 0 : _f.fileLang) || "");
    }
    onCleanup(() => {
      var _a3;
      return (_a3 = diffFile2 == null ? void 0 : diffFile2.clear) == null ? void 0 : _a3.call(diffFile2);
    });
    return diffFile2;
  };
  const [wrapperRef, setWrapRef] = createSignal(null);
  const diffFile = createMemo(getInstance);
  const [isMounted, setIsMounted] = createSignal(false);
  createEffect(() => {
    setIsMounted(true);
  });
  const [widgetState, setWidgetState] = createSignal({
    side: (_a = props.initialWidgetState) == null ? void 0 : _a.side,
    lineNumber: (_b = props.initialWidgetState) == null ? void 0 : _b.lineNumber
  });
  const reactiveHook = createDiffConfigStore(props, ((_c = diffFile()) == null ? void 0 : _c.getId()) || "");
  createEffect(() => {
    const {
      setId,
      setEnableAddWidget,
      setEnableHighlight,
      setEnableWrap,
      setExtendData,
      setFontSize,
      setIsIsMounted,
      setMode,
      setOnAddWidgetClick,
      setRenderExtendLine,
      setRenderWidgetLine
    } = reactiveHook.getReadonlyState();
    const currentDiffFile = diffFile();
    setId((currentDiffFile == null ? void 0 : currentDiffFile.getId()) || "");
    setEnableAddWidget(!!props.diffViewAddWidget);
    setEnableHighlight(!!props.diffViewHighlight);
    setEnableWrap(!!props.diffViewWrap);
    setFontSize(props.diffViewFontSize || 14);
    setIsIsMounted(isMounted());
    setMode(props.diffViewMode || DiffModeEnum.Split);
    setOnAddWidgetClick({
      current: props.onAddWidgetClick
    });
    setRenderExtendLine(props.renderExtendLine);
    setRenderWidgetLine(props.renderWidgetLine);
    setExtendData(props.extendData);
  });
  createEffect(() => {
    if (props.initialWidgetState) {
      setWidgetState({
        side: props.initialWidgetState.side,
        lineNumber: props.initialWidgetState.lineNumber
      });
    }
  });
  const initSubscribe = () => {
    const mounted = isMounted();
    const currentDiffFile = diffFile();
    if (mounted && props.diffFile && currentDiffFile) {
      props.diffFile._addClonedInstance(currentDiffFile);
      onCleanup(() => {
        var _a2;
        return (_a2 = props.diffFile) == null ? void 0 : _a2._delClonedInstance(currentDiffFile);
      });
    }
  };
  const initDiff = () => {
    const mounted = isMounted();
    const currentDiffFile = diffFile();
    if (mounted && currentDiffFile) {
      currentDiffFile.initTheme(props.diffViewTheme);
      currentDiffFile.initRaw();
      currentDiffFile.buildSplitDiffLines();
      currentDiffFile.buildUnifiedDiffLines();
    }
  };
  const initSyntax = () => {
    const mounted = isMounted();
    const currentDiffFile = diffFile();
    if (mounted && currentDiffFile && props.diffViewHighlight) {
      currentDiffFile.initSyntax({
        registerHighlighter: props.registerHighlighter
      });
      currentDiffFile.notifyAll();
    }
  };
  const initAttribute = () => {
    const mounted = isMounted();
    const currentDiffFile = diffFile();
    props.diffViewTheme;
    if (mounted && currentDiffFile && wrapperRef()) {
      const init = () => {
        var _a2, _b2;
        (_a2 = wrapperRef()) == null ? void 0 : _a2.setAttribute("data-theme", currentDiffFile._getTheme() || "light");
        (_b2 = wrapperRef()) == null ? void 0 : _b2.setAttribute("data-highlighter", currentDiffFile._getHighlighterName());
      };
      init();
      const cb = currentDiffFile.subscribe(init);
      onCleanup(() => cb());
    }
  };
  createEffect(initSubscribe);
  createEffect(initDiff);
  createEffect(initSyntax);
  createEffect(initAttribute);
  onCleanup(() => reactiveHook.clear());
  return createComponent(Show, {
    get when() {
      return diffFile();
    },
    get children() {
      var _el$ = _tmpl$2();
      use(setWrapRef, _el$);
      setAttribute(_el$, "data-version", "0.0.3");
      insert(_el$, createComponent(DiffViewContext.Provider, {
        value: reactiveHook,
        get children() {
          return createComponent(DiffWidgetContext.Provider, {
            value: [widgetState, setWidgetState],
            get children() {
              var _el$2 = _tmpl$(), _el$3 = _el$2.firstChild;
              insert(_el$3, (() => {
                var _c$ = createMemo(() => !!(!props.diffViewMode || props.diffViewMode & DiffModeEnum.Split));
                return () => _c$() ? createComponent(DiffSplitView, {
                  get diffFile() {
                    return diffFile();
                  }
                }) : createComponent(DiffUnifiedView, {
                  get diffFile() {
                    return diffFile();
                  }
                });
              })());
              createRenderEffect((_p$) => {
                var _a2;
                var _v$ = {
                  [diffFontSizeName]: (props.diffViewFontSize || 14) + "px"
                }, _v$2 = isMounted() ? `diff-root${(_a2 = diffFile()) == null ? void 0 : _a2.getId()}` : void 0, _v$3 = "diff-view-wrapper" + (props.class ? ` ${props.class}` : ""), _v$4 = props.style;
                _p$.e = style(_el$2, _v$, _p$.e);
                _v$2 !== _p$.t && setAttribute(_el$3, "id", _p$.t = _v$2);
                _v$3 !== _p$.a && className(_el$3, _p$.a = _v$3);
                _p$.o = style(_el$3, _v$4, _p$.o);
                return _p$;
              }, {
                e: void 0,
                t: void 0,
                a: void 0,
                o: void 0
              });
              return _el$2;
            }
          });
        }
      }));
      createRenderEffect((_p$) => {
        var _a2, _b2, _c2, _d;
        var _v$5 = ((_b2 = (_a2 = diffFile()) == null ? void 0 : _a2._getTheme) == null ? void 0 : _b2.call(_a2)) || "light", _v$6 = (_d = (_c2 = diffFile()) == null ? void 0 : _c2._getHighlighterName) == null ? void 0 : _d.call(_c2);
        _v$5 !== _p$.e && setAttribute(_el$, "data-theme", _p$.e = _v$5);
        _v$6 !== _p$.t && setAttribute(_el$, "data-highlighter", _p$.t = _v$6);
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$;
    }
  });
};
function SolidDiffView(props) {
  return createComponent(InternalDiffView, props);
}
const DiffView = SolidDiffView;
export {
  DiffModeEnum,
  DiffView,
  SplitSide2 as SplitSide
};
//# sourceMappingURL=solid-git-diff-view.mjs.map
